package edu.nps.seed.nst

/*
	code borrowed/modified from the Scala stairway book "Programming in Scala" by
 * M. Odersky, L. Spoon, and B. Venners 2008, pg 368
 */
abstract class Simulation {
	type Action = () => Unit
   
   case class Event(time: Double, action: Action)
 
   private var curtime = 0.
   def currentTime: Double = curtime
   
   private var schedule: List[Event] = List()
   
   private def addEvent(sched: List[Event],
                     event: Event): List[Event] = {
      if (sched.isEmpty || event.time < sched.head.time) event :: sched
      
      else sched.head :: addEvent(sched.tail, event)
   }
   
   def numEvents(): Int = {
     schedule.size
   }
   
   def scheduleEventAtTime(time: Double)(block: => Unit) {
		val event = Event(time, () => block)
		schedule = addEvent(schedule, event)
   }
   
   def clearSchedule() {
	   schedule = Nil
   }
   
   
   private def next() {
	   (schedule: @unchecked) match {
	     case event :: rest =>
	       schedule = rest
	       curtime = event.time
	       event.action()
	   }
   }
   
   def run() {

     while (!schedule.isEmpty) next()
     println(curtime+",simulation,end")
     println("*** simulation ended, time = " +
    	curtime +" ***")
     
   }
}
