package edu.nps.seed.nst

import scala.collection.mutable.Map
import scala.collection.mutable.ListBuffer

trait Statistic {
	def calculate(): List[DataPair]
 
}


class TaskStatusCount(val tasks: List[Task]) extends Statistic {
  
  def calculate(): List[DataPair] = {
    val values = new ListBuffer[DataPair]
    for (s <- TaskStatus) {
        var count = 0
	    for (task <- tasks if task.status == s) {
	    	count = count + 1
	    }      
        var name = "TaskStatus_"+s.toString
        values += new DataPair(name, count)
    }
    
    values.toList
  }
  
  
}

class ResourceStats(val resources: List[Resource]) extends Statistic {
  
  def calculate(): List[DataPair] = {
    val values = new ListBuffer[DataPair]
    for (r <- resources) {
      values += new DataPair((r.name+":UseTime"),r.useTime)
      values += new DataPair((r.name+":OnTestTime"),r.onTestTime)
      values += new DataPair((r.name+":NumTasks"),r.numTasks)
      
    }
    
    values.toList
  }
  
  
}

class TotalTestTime() extends Statistic {
  
  def calculate(): List[DataPair] = {
    val values = new ListBuffer[DataPair]
	val totalTime = TestEvent.testEndTime - TestEvent.testStartTime
	values += new DataPair("TotalTestTime",totalTime)
 
	values.toList
    
  }
}


