package oldmcdata.agents;

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.SimpleBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import oldmcdata.evaluators.Evaluator;
import oldmcdata.evaluators.EvaluatorFactory;


/**
 * Title:        EvaluatorTest<p>
 * Description:  A simple agent that can send  custom messages to another agent.  Based on AgentReceiver in
 * JADE examples.receivers.AgentReceiver.<p>
 *Copyright: 2000 - 2005
 *         <p>
 * @author Stephen C. Upton
 * @version $Date: 12/4/2000  $ $Revision: 1.0 $
 *
 *
 */
public class EvaluatorAgent extends OldMcDataAgent {
    private Evaluator evaluator;
    private boolean initialRequest = true;
    private long timeToWaitToInitialize = 5000;

    protected void setup() {
        super.setup();
        this.addBehaviour(new WaitForEvaluationRequest(this));
        System.out.println("this is a test of the EvaluatorAgent arguments list: "
                + "StudyDir: " + studyDirName + " StudyFile: " + studyFileName);
        addBehaviour(new WaitForRunsCompletedMessage(this));
    }

    class InitialBehavior extends SimpleBehaviour {
        private boolean finished = false;

        public InitialBehavior(Agent a) {
            super(a);
        }

        public void action() {
        	myAgent.doWait(timeToWaitToInitialize);
            doInitialization();
            myAgent.addBehaviour(new DoEvaluationBehavior(myAgent));
            finished = true;
        }

        private void doInitialization() {
            System.out.println("\nAgent " + myAgent.getLocalName() + "  is in InitialBehavior");
            output.println("\nAgent " + myAgent.getLocalName() + "  is in InitialBehavior");
            evaluator = EvaluatorFactory.getInstance(studyDirName, studyFileName);
            evaluator.setConfigFile(configFileName);
            evaluator.initialize();
        }

        public boolean done() {
            return finished;
        }
    }   // end InitialBehavior

    class WaitForEvaluationRequest extends SimpleBehaviour {
        private boolean finished = false;
        private String message = "RequestEvaluate";

        public WaitForEvaluationRequest(Agent a) {
            super(a);
        }

        public void action() {
            System.out.println("\nAgent " + myAgent.getLocalName() + "  is waiting for message: " + message);
            output.println("\nAgent " + myAgent.getLocalName() + "  is waiting for message: " + message);
//        try {
            MessageTemplate m1 = MessageTemplate.MatchOntology("GEA");

            //The agent waits for a specific message. If it doesn't arrive the behaviour is suspended until a new message arrives.
            ACLMessage msg = myAgent.receive(m1);

            if (msg != null) {
                System.out.println("\nAgent " + myAgent.getLocalName() + " received message: " + message);
                output.println("\nAgent " + myAgent.getLocalName() + " received message: " + message);
                output.println(msg.toString());
                System.out.println(msg.toString());
                //        ContentElement content = myAgent.getContentManager().extractContent(msg);
                //       Object resp = l.get(0);
                //				List msgContent = myAgent.extractContent(msg);
                finished = true;
                if (initialRequest) {
                    myAgent.addBehaviour(new InitialBehavior(myAgent));
                    initialRequest = false;
                } else
                    myAgent.addBehaviour(new DoEvaluationBehavior(myAgent));
            } else {
                output.println("\nAgent " + myAgent.getLocalName() + " has not received an RequestEvaluate message");
                block();
            }
//        } // End of try
//        catch (FIPAException fe) {
//                System.err.println("FIPAException in fill/extract content:" + fe.getMessage());
//                System.exit(-1);
//        }
        }

        public boolean done() {
            return finished;
        }
    }  // end WaitForRequestBehavior

    class DoEvaluationBehavior extends SimpleBehaviour {
        private boolean finished = false;

        public DoEvaluationBehavior(Agent a) {
            super(a);
        }

        public void action() {
            doEvaluation();
            finished = true;
        }

        private void doEvaluation() {
            output.println("Doing the Simulation");
            evaluator.evaluate();
            Behaviour behavior = new EvaluationCompleteBehavior(myAgent);
            evaluator.scheduleAgentsNextBehavior(myAgent, behavior);
        }

        public boolean done() {
            return finished;
        }
    }  // end DoEvaluationBehavior


    class EvaluationCompleteBehavior extends SimpleBehaviour {
        private boolean finished = false;

        public EvaluationCompleteBehavior(Agent a) {
            super(a);
        }

        public void action() {
            System.out.println("\nAgent " + myAgent.getLocalName() + "  has completed Evaluation");
            output.println("\nAgent " + myAgent.getLocalName() + "  has completed Evaluation");
            sendEvaluationCompleteMessage();
            myAgent.addBehaviour(new WaitForEvaluationRequest(myAgent));
            finished = true;
        }

        private void sendEvaluationCompleteMessage() {
            AID r = new AID();
            r.setLocalName("generator");
            ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
            msg.setSender(myAgent.getAID());
            msg.addReceiver(r);
            msg.setContent("CompletedExcursions");
            msg.setLanguage("PlainText");
            msg.setOntology("GEA");
            send(msg);
        }

        public boolean done() {
            return finished;
        }
    } // end EvaluationCompleteBehavior

    class WaitForRunsCompletedMessage extends SimpleBehaviour {
        private boolean finished = false;
        private String message = "WaitForRunsCompletedMessage";

        public WaitForRunsCompletedMessage(Agent a) {
            super(a);
        }

        public void action() {
            output.println("\nAgent " + myAgent.getLocalName() + "  is waiting for message: " + message);
            System.out.println("\nAgent " + myAgent.getLocalName() + "  is waiting for message: " + message);
            MessageTemplate m1 = MessageTemplate.MatchOntology("RunsCompleted");

            //The agent waits for a specific message. If it doesn't arrive the behaviour is suspended until a new message arrives.
            ACLMessage msg = myAgent.receive(m1);

            if (msg != null) {
                output.println("\nAgent " + myAgent.getLocalName() + " is outa here!!");
                System.out.println("\nAgent " + myAgent.getLocalName() + " is outa here!!");
                finished = true;
                output.close();
                myAgent.doDelete();
            } else {
                block();
            }
        }

        public boolean done() {
            return finished;
        }
    }     // end WaitForRunsCompletedMessage

}

