package oldmcdata.agents;

import jade.core.Agent;
import jade.core.AgentManager;
import jade.core.ContainerID;
import jade.core.behaviours.SimpleBehaviour;
import jade.domain.FIPAException;
import jade.domain.JADEAgentManagement.CreateAgent;
import jade.domain.JADEAgentManagement.JADEAgentManagementOntology;
import jade.domain.JADEAgentManagement.KillContainer;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.lang.sl.SL0Codec;
import jade.onto.basic.Action;
import jade.util.leap.ArrayList;
import jade.util.leap.List;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import oldmcdata.utils.CopyFilesFromTo;
import oldmcdata.utils.DefaultStudyXMLHandler;
import oldmcdata.utils.StudyXMLHandler;

/**
 * Title:        StarterAgent
 * Description:
 *Copyright: 2000 - 2005
 * @author  Stephen C. Upton
 * @version 1.0
 */

public class StarterAgent extends Agent {
    private PrintWriter output;
    private String studyFileName;
    private String studyDirName;
    private String configFileName;

    protected void setup() {
        try {
            Object[] args = this.getArguments();
            studyDirName = (String) args[0];
            studyFileName = (String) args[1];
            configFileName = (String) args[2];
            // make a copy of the study.xml file
            String origStudyXMLFileName = "orig-" + studyFileName;
            File studyXMLFile = new File(studyDirName, studyFileName);
            File origStudyXMLFile = new File(studyDirName, origStudyXMLFileName);
            CopyFilesFromTo.copyFromTo(studyXMLFile, origStudyXMLFile);
            // try parsing the study.xml file to see if in correct format -
            // done here to prevent further unnecessary processing by the other agents.
            StudyXMLHandler xmlHandler = new DefaultStudyXMLHandler(studyDirName, studyFileName);
            xmlHandler.openStudyXMLDocument();
            output = new PrintWriter(new BufferedWriter(
                    new FileWriter(studyDirName + File.separator + "starter.log")));

            // Register the codec for the SL0 language
            registerLanguage(SL0Codec.NAME, new SL0Codec());
            // Register the ontology
            this.registerOntology(JADEAgentManagementOntology.NAME,
                    JADEAgentManagementOntology.instance());
            this.addBehaviour(new CreateAgentsBehavior(this));

        } catch (IOException ioe) {
            ioe.printStackTrace(System.out);
            System.exit(-1);
        }
    }

    class CreateAgentsBehavior extends SimpleBehaviour {
        private boolean finished = false;

        public CreateAgentsBehavior(Agent a) {
            super(a);
        }

        public void action() {
            System.out.println("\nAgent " + myAgent.getLocalName() + "  is in CreateAgentsBehavior");
            output.println("\nAgent " + myAgent.getLocalName() + "  is in CreateAgentsBehavior");
            this.createAgent(OldMcDataAgent.generator, "oldmcdata.agents.GeneratorAgent",
                    studyDirName, studyFileName, configFileName);
            this.createAgent(OldMcDataAgent.evaluator, "oldmcdata.agents.EvaluatorAgent",
                    studyDirName, studyFileName, configFileName);
            this.createAgent(OldMcDataAgent.analyzer, "oldmcdata.agents.AnalyzerAgent",
                    studyDirName, studyFileName, configFileName);
            myAgent.addBehaviour(new WaitForRunsCompletedMessage(myAgent));
            finished = true;
            output.close();

        }

        private void createAgent(String newAgentName, String className,
                                 String studyDir, String fileName, String configFileName) {
            try {
                CreateAgent ca = new CreateAgent();
                ca.setAgentName(newAgentName);
                ca.setClassName(className);
                ca.addArguments(studyDir);
                ca.addArguments(fileName);
                ca.addArguments(configFileName);
                ca.setContainer(new ContainerID(AgentManager.MAIN_CONTAINER_NAME, null));

                Action action = new Action();
                action.set_0(getAMS());
                action.set_1(ca);
                List content = new ArrayList(1);
                content.add(action);

                ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
                msg.setSender(myAgent.getAID());
                msg.addReceiver(myAgent.getAMS());
                msg.setLanguage(SL0Codec.NAME);
                msg.setOntology(JADEAgentManagementOntology.NAME);
                myAgent.fillMsgContent(msg, content);

                myAgent.addBehaviour(new OldMcDataAgentAMSClientBehavior(myAgent, msg, ca.getClass().getName()));
                System.out.println("\nAgent " + myAgent.getLocalName() + " has sent request to create agent " +
                        newAgentName);

                output.println("\nAgent " + myAgent.getLocalName() + " has sent request to create agent " +
                        newAgentName);
                output.println(msg.toString());

            } catch (FIPAException fe) {
                System.err.println("FIPAException in CreateAgentsBehavior: " + fe.getMessage());
                fe.printStackTrace(System.out);
            }
        }

        public boolean done() {
            return finished;
        }

    } // end CreateAgentsBehavior

    class WaitForRunsCompletedMessage extends SimpleBehaviour {
        private boolean finished = false;

        public WaitForRunsCompletedMessage(Agent a) {
            super(a);
        }

        public void action() {
            output.println("\nAgent " + myAgent.getLocalName() + "  is WaitingForRunsCompletedMessage");
            MessageTemplate m1 = MessageTemplate.MatchOntology("RunsCompleted");

            //The agent waits for a specific message. If it doesn't arrive the behaviour is suspended until a new message arrives.
            ACLMessage msg = myAgent.receive(m1);

            if (msg != null) {
                myAgent.addBehaviour(new ShutDownPlatformBehavior(myAgent));
            } else {
                block();
            }
        }

        public boolean done() {
            return finished;
        }
    }     // end WaitForRunsCompletedMessage


    class ShutDownPlatformBehavior extends SimpleBehaviour {
        private boolean finished = false;

        public ShutDownPlatformBehavior(Agent a) {
            super(a);
        }

        public void action() {
            System.out.println("\nAgent " + myAgent.getLocalName() + "  is in ShutDownPlatform");
            output.println("\nAgent " + myAgent.getLocalName() + "  is in ShutDownPlatform");
            this.sendShutDownRequest();
            finished = true;
            output.close();

        }

        private void sendShutDownRequest() {
            try {
                KillContainer kc = new KillContainer();
                kc.setContainer(new ContainerID(AgentManager.MAIN_CONTAINER_NAME, null));

                Action action = new Action();
                action.set_0(getAMS());
                action.set_1(kc);
                List content = new ArrayList(1);
                content.add(action);

                ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
                msg.setSender(myAgent.getAID());
                msg.addReceiver(myAgent.getAMS());
                msg.setLanguage(SL0Codec.NAME);
                msg.setOntology(JADEAgentManagementOntology.NAME);
                myAgent.fillMsgContent(msg, content);

                myAgent.addBehaviour(new OldMcDataAgentAMSClientBehavior(myAgent, msg, kc.getClass().getName()));
                output.println("\nAgent " + myAgent.getLocalName() + " has sent request for Shutdown");
                output.println(msg.toString());

            } catch (Exception fe) {
                System.err.println("Exception in ShutDownPlatformBehavior: " + fe.getMessage());
                fe.printStackTrace(System.out);
            }
        }

        public boolean done() {
            return finished;
        }

    } // end ShutDownPlatformBehavior
}