/*
    
    oldmcdata1.0
    Copyright (C) 2005 Referentia Systems, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package oldmcdata.analyzers.utils;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.List;
import java.util.Vector;

import org.dom4j.Element;

/**
 * @author Stephen C. Upton
 * @date Feb 22, 2005
 *
 */
public class DataReader {
    private List startsWithList, linesRead;
    private int numberOfLinesToSkip = 0, lineCount = 0;
    private OutputDataFileHandler outputDataFileHandler;
    private BufferedReader dataReader;
    
    public DataReader() {
        startsWithList = new Vector();
        linesRead = new Vector();
        outputDataFileHandler = new OutputDataFileHandler();

    }
    
    public void initialize(Element e) {
        Element skipLines = e.element("SkipLines");
        String numberOfLinesToSkipValue = skipLines.elementText("Number");
        if ( numberOfLinesToSkipValue != null )
            numberOfLinesToSkip = Integer.parseInt(numberOfLinesToSkipValue);
        List startsWithElements = skipLines.elements("StartsWith");
        for (int i=0;i<startsWithElements.size();i++) {
            Element startsWithElement = (Element)startsWithElements.get(i);
            String startsWithValue = startsWithElement.getTextTrim();
            if ( startsWithValue != null )
                startsWithList.add(startsWithValue);
        }
    }
    
    public void setFileToProcess(String filename) {
        dataReader = outputDataFileHandler.getDataReader(filename);
    }
    
    public void readLines() {
        linesRead.clear();
		lineCount = 0;
		String lineRead = "";
		
		boolean eof = false;
		while (!eof) {
			try {
				lineRead = dataReader.readLine();
				lineCount++;
				if ( lineRead == null ) {
					eof = true;
					break;					
				}
				// first skip the number of lines specified by Number in SkipLines
				if ( lineCount <= numberOfLinesToSkip ) {
				    continue;
				}
	
				// now skip those that match the constraints
				if ( lineMatches(lineRead) ) {
				    continue;
				}
				// if the lineRead doesn't match any of the contraints, then add it	
				linesRead.add(lineRead);

			}
			catch(IOException ioe) {
				System.out.println("Had trouble reading line: " + lineCount);
				System.out.println(lineRead);
			
			}			
		}
    }
    
    private boolean lineMatches(String lineToProcess) {
        for (int i=0;i<startsWithList.size();i++) {
            String startsWithValue = (String)startsWithList.get(i);
            if ( lineToProcess.startsWith(startsWithValue)) 
                return true;
        }
        
        return false;
    }
    
    public List getLinesRead() {
        return linesRead;
    }
    
}
