package oldmcdata.evaluators.condor;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.List;

import oldmcdata.evaluators.JobRunner;

import org.dom4j.Element;


/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class CondorJobRunner implements JobRunner {
    private String[] cmdarray;

    public CondorJobRunner() {
    }

    public void run(List cmdLines) {
        Iterator cmds = cmdLines.iterator();
        while (cmds.hasNext()) {
            Object commandLine = cmds.next();
            if (commandLine instanceof List)
                constructCommandLine((List) commandLine);
            if (commandLine instanceof Element)
                constructCommandLine((Element) commandLine);
            runCommandLine();
        }
    }

    /** handles a Element of a List of Elements as the command line arguments
     */
    private void constructCommandLine(Element cmdLine) {
        List arguments = cmdLine.elements("argument");
        cmdarray = new String[arguments.size()];
        for (int i = 0; i < arguments.size(); i++) {
            Element argument = (Element) arguments.get(i);
            cmdarray[i] = argument.getTextTrim();
        }
    }

    /** handles a List of a List of Strings as the command line arguments
     */
    private void constructCommandLine(List arguments) {
        cmdarray = new String[arguments.size()];
        for (int i = 0; i < arguments.size(); i++) {
            cmdarray[i] = (String) arguments.get(i);
        }
    }


    public void runCommandLine() {
		boolean err = false;
        System.out.println("Command Line: ");
        for (int i = 0; i < cmdarray.length; i++)
            System.out.print(cmdarray[i] + " ");
        System.out.println();
   		
        try {
        	Process pp = new ProcessBuilder(cmdarray).start();
        	BufferedReader results = new BufferedReader(
        			new InputStreamReader(pp.getInputStream()));
        	String s;
        	while ((s = results.readLine()) != null)
        		System.out.println(s);
        	BufferedReader errors = new BufferedReader(
        			new InputStreamReader(pp.getErrorStream()));
        	while ((s = errors.readLine()) != null ) {
        		System.err.println(s);
        		err = true;
        	}
        	pp.waitFor();

        } catch (Exception ex) {
            ex.printStackTrace(System.out);
            System.exit(-1);
        }
        
		if (err)
			throw new RuntimeException("Errors executing command: " + cmdarray);            
    }

}