package oldmcdata.evaluators.condor;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.List;

import oldmcdata.utils.DefaultStudyXMLHandler;
import oldmcdata.utils.StudyXMLHandler;

import org.apache.oro.text.regex.MalformedPatternException;
import org.apache.oro.text.regex.Pattern;
import org.apache.oro.text.regex.Perl5Compiler;
import org.apache.oro.text.regex.Perl5Matcher;
import org.apache.oro.text.regex.StringSubstitution;
import org.apache.oro.text.regex.Util;


/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class CondorNodeRunner {
    private String masterCommandLine, repCommandLine;
    private int numberOfReplications, numberOfSeeds;
    private List seeds;
    private StringSubstitution substitution;
    private Perl5Matcher matcher;
    private Pattern pattern;

    public CondorNodeRunner(String studyFileName, String numberOfReps, String commandLine) {
        masterCommandLine = commandLine;
        numberOfReplications = new Integer(numberOfReps).intValue();
        StudyXMLHandler studyXMLHandler = new DefaultStudyXMLHandler(".", studyFileName);
        studyXMLHandler.openStudyXMLDocument();
        seeds = studyXMLHandler.getInitialSeeds();
        numberOfSeeds = seeds.size();
        substitution = new StringSubstitution();
        matcher = new Perl5Matcher();
        Perl5Compiler compiler = new Perl5Compiler();
        try {
            pattern = compiler.compile("\\$seed");
        } catch (MalformedPatternException mpe) {
            System.err.println("Can't find pattern $seed in java command line used by CondorNodeRunner");
            System.err.println("command line: " + commandLine);
            System.err.println("error is likely due to batch file that started CondorNodeRunner");
            System.exit(-1);
        }
    }

    public void run() {
        if (numberOfSeeds < numberOfReplications)
            addSeeds();
        for (int i = 0; i < numberOfReplications; i++) {
            constructCommandLine(i);
            runCommandLine();
        }
    }

    private void addSeeds() {
        String lastSeed = (String) seeds.get(numberOfSeeds - 1);
        long seed = new Long(lastSeed).longValue();
        for (int i = numberOfSeeds; i < numberOfReplications; i++)
            seeds.add(i, Long.toString(seed + i + 1));
    }

    private void constructCommandLine(int repNum) {
        String seed = (String) seeds.get(repNum);
        substitution.setSubstitution(seed);
        //replace $seed in masterCommandLine with this actual seed
        repCommandLine = Util.substitute(matcher, pattern, substitution, masterCommandLine);
    }


    public void runCommandLine() {
		boolean err = false;
        System.out.println("Command Line: ");
        System.out.print(repCommandLine);
        System.out.println();
   		
        try {
        	Process pp = new ProcessBuilder(repCommandLine.split(" ")).start();
        	BufferedReader results = new BufferedReader(
        			new InputStreamReader(pp.getInputStream()));
        	String s;
        	while ((s = results.readLine()) != null)
        		System.out.println(s);
        	BufferedReader errors = new BufferedReader(
        			new InputStreamReader(pp.getErrorStream()));
        	while ((s = errors.readLine()) != null ) {
        		System.err.println(s);
        		err = true;
        	}
        	pp.waitFor();

        } catch (Exception ex) {
            ex.printStackTrace(System.out);
            System.exit(-1);
        }
        
		if (err)
			throw new RuntimeException("Errors executing command: " + repCommandLine); 

    }

    public static void main(String[] args) {
        String studyFileName = args[0];
        String numberOfReplications = args[1];
        String commandLine = args[2];
        CondorNodeRunner runner = new CondorNodeRunner(studyFileName, numberOfReplications, commandLine);
        runner.run();

    }

}