package oldmcdata.evaluators.local;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import oldmcdata.evaluators.JobRunner;
import oldmcdata.utils.CopyFilesFromTo;

import org.dom4j.Element;


/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class LocalJobRunner implements JobRunner {
    private String[] cmdarray;
    private File studyDirectory, outputDirectory, runDirectory;
	private List<File>  outputFiles;
	private Set<File> inputRunFiles;

    public LocalJobRunner() {
		inputRunFiles = new HashSet<File>();
		outputFiles = new ArrayList<File>();
    }
    
    public void runLocalJobs(List<LocalJob> localJobs) {
    	createDirectories();
    	for (LocalJob job : localJobs) {
    		transferInputFiles(job);
    		job.setRunDirectory(runDirectory);
    		job.run( );
    		transferOutputFiles();
    	}
    }
    
    private void createDirectories() {
    	runDirectory = new File(studyDirectory, "run");
    	if ( !runDirectory.exists() ) {
    		runDirectory.mkdir();
    	}
    	if ( !outputDirectory.exists() ) {
    		outputDirectory.mkdir();
    	}
    }
    
	private void transferInputFiles(LocalJob job) {
		for (File inputFile : job.getInputTransferFiles()) {
			File inputInRun = new File(runDirectory, inputFile.getName());
			inputRunFiles.add(inputInRun);
			if ( !inputInRun.exists() ) {
				CopyFilesFromTo.copyFromTo(inputFile, inputInRun);
			}
		}
	}
	
	private void transferOutputFiles() {
		outputFiles.clear();
		System.out.println("Copying output files to: " + outputDirectory.toString());
		File[] runFiles = runDirectory.listFiles();
		//only copy output files to the output directory and not the input files
		for (File runFile : runFiles) {
			if ( !inputRunFiles.contains(runFile) ) {
				CopyFilesFromTo.copyFromTo(runFile, new File(outputDirectory, runFile.getName()));
				outputFiles.add(runFile);
			}
		}
		// now delete all the output files
		System.out.println("Deleting output files");
		for (File outFile : outputFiles) {
			if (outFile.delete() ) {
				System.out.println("deleted " + outFile.toString());
			}
			else {
				System.out.println("could not delete " + outFile.toString());
			}
		}
		
		
	}
    
    public void setStudyDirectory(File studydir) {
    	studyDirectory = studydir;
    }
    
    public void setOutputDirectory(File outdir) {
    	outputDirectory = outdir;
    }

    public void run(List cmdLines) {
        Iterator cmds = cmdLines.iterator();
        while (cmds.hasNext()) {
            Object commandLine = cmds.next();
            if (commandLine instanceof List)
                constructCommandLine((List) commandLine);
            if (commandLine instanceof Element)
                constructCommandLine((Element) commandLine);
            runCommandLine();
        }
    }

    /** handles a Element of a List of Elements as the command line arguments
     */
    private void constructCommandLine(Element cmdLine) {
        List arguments = cmdLine.elements("argument");
        cmdarray = new String[arguments.size()];
        for (int i = 0; i < arguments.size(); i++) {
            Element argument = (Element) arguments.get(i);
            cmdarray[i] = argument.getTextTrim();
        }
    }

    /** handles a List of a List of Strings as the command line arguments
     */
    private void constructCommandLine(List arguments) {
        cmdarray = new String[arguments.size()];
        for (int i = 0; i < arguments.size(); i++) {
            cmdarray[i] = (String) arguments.get(i);
        }
    }


    public void runCommandLine() {
		boolean err = false;
        System.out.println("Command Line: ");
        for (int i = 0; i < cmdarray.length; i++)
            System.out.print(cmdarray[i] + " ");
        System.out.println();
   		
        try {
        	Process pp = new ProcessBuilder(cmdarray).start();
        	BufferedReader results = new BufferedReader(
        			new InputStreamReader(pp.getInputStream()));
        	String s;
        	while ((s = results.readLine()) != null)
        		System.out.println(s);
        	BufferedReader errors = new BufferedReader(
        			new InputStreamReader(pp.getErrorStream()));
        	while ((s = errors.readLine()) != null ) {
        		System.err.println(s);
        		err = true;
        	}
        	pp.waitFor();

        } catch (Exception ex) {
            ex.printStackTrace(System.out);
            System.exit(-1);
        }
        
		if (err)
			throw new RuntimeException("Errors executing command: " + cmdarray); 

    }

}