package oldmcdata.generators;


import java.util.ArrayList;
import java.util.List;

import nalex.algorithms.Algorithm;
import nalex.operators.init.Initialize;
import nalex.operators.select.Select;
import nalex.operators.stop.Stop;
import nalex.operators.transform.Transform;
import nalex.populations.Individual;
import nalex.populations.Population;
import oldmcdata.generators.utils.AlgorithmConfigurationManager;
import oldmcdata.generators.utils.NALEXModelEvaluate;
import oldmcdata.generators.utils.PopulationWriter;

import org.dom4j.Element;


/**
 * Title: EvolutionaryProgrammingGenerator
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.3
 */

public class EvolutionaryProgrammingGenerator extends NaturalHeuristicGenerator {
    private final static String FITNESS = "Fitness";
    private List steps;
    private PopulationWriter popWriter;
// variables associated with NALEX
    private Algorithm algorithm;
    private Population currentPopulationUnderEvaluation;
    private Population parentPopulation;
    private Population offspringPopulation;


    public EvolutionaryProgrammingGenerator() {
        super();
        steps = new ArrayList();
    }

    public void configure() {
        super.configure();
        AlgorithmConfigurationManager configManager = new AlgorithmConfigurationManager();
        configManager.configure(studyXMLHandler.getStudyDirectory(),
                studyXMLHandler.getGeneratorAlgorithmParameters(),
                studyXMLHandler.getVariables());
        algorithm = configManager.getAlgorithm();
        popWriter = configManager.getPopulationWriter();
        // first 3 steps of the algorithm
        steps.add(new InitializerStep());
        steps.add(new EvaluatorStep());
        steps.add(new TransformerStep());
    }

    public boolean generate() {
        excursions = new ArrayList();
        boolean doNextStep = true;
        System.out.println(this.getClass().getName() + ":generate()");

        while (doNextStep) {
            AlgorithmStep step = (AlgorithmStep) steps.remove(0);
            doNextStep = step.execute();
        }

        return stop; // check for stopping conditions in Transformer below
    }

    private void generateExcursions(Population pop) {
        for (int i = 0; i < pop.size(); i++) {
            Individual nalexIndividual = pop.getIndividual(i);
            excursions.add(nalexIndividual.getPoint().toString());
        }
    }

    class InitializerStep implements AlgorithmStep {
        public boolean execute() {
            studyXMLHandler.openStudyXMLDocument();
            initializeIndividuals();
            translator.translate((int)iterationCount, excursions, studyXMLHandler);
            studyXMLHandler.writeStudyXMLDocument();
            writeRenamedBaseCase();
            return false;
        }

        public void initializeIndividuals() {
            Initialize initialize = null;
//            try {
                initialize = (Initialize)algorithm.getOperator("init");
                parentPopulation = algorithm.getPopulation("parent");
                initialize.initialize(parentPopulation);
                parentPopulation.setGeneration(iterationCount);
                currentPopulationUnderEvaluation = parentPopulation;
                generateExcursions(parentPopulation);
//            } catch (NoSuchMethodError e) {
//                System.out.println("initialize: " + initialize.getClass().getName());
//                e.printStackTrace();  //To change body of catch statement use Options | File Templates.
//            }
        }
    }

    class TransformerStep implements AlgorithmStep {
        public boolean execute() {
            studyXMLHandler.openStudyXMLDocument();
            transformIndividuals();
            incrementIterations();
            if (stop) return false;
            translator.translate((int)iterationCount, excursions, studyXMLHandler);
            studyXMLHandler.writeStudyXMLDocument();
            writeRenamedBaseCase();
            steps.add(new EvaluatorStep());
            steps.add(new SelectorStep());
            return false;
        }

        void transformIndividuals() {
            Transform transform = (Transform)algorithm.getOperator("transform");
            offspringPopulation = transform.transform(parentPopulation);
            offspringPopulation.setGeneration(iterationCount);
            currentPopulationUnderEvaluation = offspringPopulation;
            generateExcursions(offspringPopulation);
        }

        void incrementIterations() {
            Stop stopOp = (Stop)algorithm.getOperator("stop");
            stopOp.setOperatorCount(iterationCount);
            iterationCount++;
            stop = stopOp.stop();
        }

    }

    class EvaluatorStep implements AlgorithmStep {
        public boolean execute() {
            updateIndividuals();
            popWriter.writePopulation(currentPopulationUnderEvaluation, "offspring");
            return true;
        }

        void updateIndividuals() {
            studyXMLHandler.openStudyXMLDocument();
            List excursionsList = studyXMLHandler.getMostRecentExcursions();
            List fitnesses = new ArrayList();
            for (int i = 0; i < excursionsList.size(); i++) {
                Element excursion = (Element) excursionsList.get(i);
                fitnesses.add(excursion.element(FITNESS).getTextTrim());
            }
            NALEXModelEvaluate evaluator = (NALEXModelEvaluate) algorithm.getOperator("evaluate");
            evaluator.updateFitnesses(currentPopulationUnderEvaluation, fitnesses);
        }
    }

    class SelectorStep implements AlgorithmStep {
        public boolean execute() {
            Select select = (Select)algorithm.getOperator("select");
            parentPopulation = select.select(parentPopulation, offspringPopulation);
            parentPopulation.setGeneration(iterationCount);
            popWriter.writePopulation(parentPopulation, "parents");
            steps.add(new TransformerStep());
            return true;
        }

    }

}





