package oldmcdata.generators;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.dom4j.Element;


/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.3
 */

/** A class to test implementation of Generators
 */
public class RunDataFromFileGenerator extends AbstractGenerator {
    private final static String VALUEDELIMITER = "ValueDelimiter",
    FILENAME = "FileName",
    NUMBEROFLINESTOSKIP = "NumberOfLinesToSkip",
    COMMENTDELIMITER = "CommentDelimiter",
    DEFAULTVALUEDELIMITER = ",",
    DEFAULTCOMMENTDELIMITER = "#",
    DEFAULTNUMBEROFLINESTOSKIP = "0";
    private List excursions;
    private String valueDelimiter, commentDelimiter;
    private File dataFile;
    private BufferedReader data;
    private int numberOfVariables, numberOfLinesToSkip;
    private boolean generatedExcursions = false;

    public RunDataFromFileGenerator() {
        super();
        // defaults
        valueDelimiter = DEFAULTVALUEDELIMITER;
        commentDelimiter = DEFAULTCOMMENTDELIMITER;
        setNumberOfLinesToSkip(DEFAULTNUMBEROFLINESTOSKIP);
    }

    public void initialize() {
        System.out.println(this.getClass().getName() + ":initialize()");
        translator.initialize();
        try {
            configure();
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(-1);
        }
    }

    private void configure() {
        Element parameters = studyXMLHandler.getGeneratorAlgorithmParameters();
        Element fileNameElement = parameters.element(FILENAME);
        Element delimiterElement = parameters.element(VALUEDELIMITER);
        Element numberOfLinesToSkipElement = parameters.element(NUMBEROFLINESTOSKIP);
        Element commentDelimiterElement = parameters.element(COMMENTDELIMITER);
        if (fileNameElement != null)
            setFileName(fileNameElement.getText());
        else
            throw new RuntimeException("Data file name not specified in " + this.getClass().getName());

        if (delimiterElement != null)
            setValueDelimiter(delimiterElement.getText());
        // else, leave at default of DEFAULTVALUEDELIMITER
        if (numberOfLinesToSkipElement != null)
            setNumberOfLinesToSkip(numberOfLinesToSkipElement.getTextTrim());
        if (commentDelimiterElement != null)
            setCommentDelimiter(commentDelimiterElement.getTextTrim());
    }

    public void setFileName(String noe) {
    	File studyDir = studyXMLHandler.getStudyDirectory();
        dataFile = new File(studyDir, noe);
    }

    public void setValueDelimiter(String d) {
        valueDelimiter = d;
    }

    public void setNumberOfLinesToSkip(String d) {
        numberOfLinesToSkip = Integer.parseInt(d);
    }

    public void setCommentDelimiter(String d) {
        commentDelimiter = d;
    }

    public boolean generate() {
        if (!generatedExcursions) {
            System.out.println(this.getClass().getName() + ":generate()");

            studyXMLHandler.openStudyXMLDocument();
            generateExcursions();

            translator.translate(excursions, studyXMLHandler);
            studyXMLHandler.writeStudyXMLDocument();
            writeRenamedBaseCase();
            generatedExcursions = true;
            return false;
        }

        return true;

    }

    private void generateExcursions() {
        try {
            List variables = studyXMLHandler.getVariables();
            numberOfVariables = variables.size();
            // set the translator's delimiter to be the same as the incoming data
            data = new BufferedReader(new FileReader(dataFile));
            translator.setDelimiter(valueDelimiter);
            excursions = new ArrayList();
            int linecount = 1;
            int skipLinesCount = 0;
            while (true) {
                String dataline = data.readLine();
                if (dataline == null) break;
                if (skipLinesCount++ < numberOfLinesToSkip) continue;
                if (dataline.startsWith(commentDelimiter)) continue;
                excursions.add(dataline);
                validateData(dataline, linecount);
                linecount++;
            }
        } catch (IOException ioe) {
            System.out.println("Error attempting to open: " + dataFile.toString());
            ioe.printStackTrace(System.out);
            System.exit(-1);
        } catch (InvalidDataException ide) {
            ide.printStackTrace(System.out);
            System.exit(-1);
        }
    }

    private void validateData(String dataline, int linecount) throws InvalidDataException {
        StringTokenizer st = new StringTokenizer(dataline, valueDelimiter);
        int numTokens = st.countTokens();
        if (numTokens != numberOfVariables)
            throw new InvalidDataException("line number: " + Integer.toString(linecount) +
                    " number of inputs: " + Integer.toString(numTokens) +
                    " does not match the number of variables: " + numberOfVariables);
    }

}
