/**
 * 
 */
package oldmcdata.model.pythagoras;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;


/**
 * @author stephen upton
 * @date 14 Dec 2007
 *
 */
public class RunPythagoras {
	ArrayList<String> cmdarray;
	ArrayList<String> randomSeeds;
	String studyXMLFileName, scenarioFileName, moeFileName;
	int numberOfReps = 1;
	String pathSeparator, javaHome;
	MyURLClassLoader loader;
	URL pythURL, jaxbURL;
	URL[] urls;
    private Random random = new Random();
	
	public RunPythagoras() {
		cmdarray = new ArrayList<String>();
		randomSeeds = new ArrayList<String>();
		
	}
	
	public void makeRuns() {
//	 pathSeparator = System.getProperty("path.separator");
//	 javaHome = System.getProperty("java.home");
//	 String userDir = System.getProperty("user.dir");
//	 System.out.println("UserDir: "+ userDir);
//	 String pythJar = userDir + "/pyth1_10_5.jar";
//	 String jaxbJar = userDir + "/jaxb-rt-1.0-ea.jar";
//	 //construct the URLs to the locations of the jar files containing Pythagoras and jaxb
//	 try {
//		 pythURL = new URL("file", "", pythJar);
//		 jaxbURL = new URL("file", "", jaxbJar);
//		 urls = new URL[]{pythURL, jaxbURL};
//		 System.out.println("URL: " + pythURL.toString());
//
//	 }
// 	catch (MalformedURLException  cnf) {
//		System.out.println("MalformedURLException: pythURL: " + pythURL.toString());
//		System.out.println("MalformedURLException: jaxbURL: " + jaxbURL.toString());
//		System.exit(-1);
//	}

	 openStudyXMLFile();
	 writeSeeds();
//		for (int i = 0; i< numberOfReps; i++) {
//			runPythagoras(i);
//
//			System.out.println("completed run: " + i);
//			cmdarray.clear();
//		}

    	System.out.println("completed");
		
	}
	
	@SuppressWarnings("unchecked")
	public void openStudyXMLFile() {
		try {
			SAXReader xmlReader = new SAXReader();
			Document studyDoc = xmlReader.read(studyXMLFileName);
			Element rootElement = studyDoc.getRootElement();
			Element modelParameters = rootElement.element("ModelParameters");
			String numberOfRepsString = modelParameters.elementText("NumberReplicates");
			setNumberOfReplications(numberOfRepsString);
			List<Element> seeds = modelParameters.elements("InitialRandomSeed");
			setSeeds(seeds);
		} 

		catch (DocumentException de) {
			System.out.println("Error attempting to parse: " + studyXMLFileName);
			de.printStackTrace(System.out);
			System.exit(-1);
		}
	}
	
	
    private void setSeeds(List<Element> seeds) {
    	int numberOfSeeds = 0;
    	if ( seeds == null ) {
    		addSeeds();
    		return;
    	}
    	 
    	numberOfSeeds = seeds.size();
    	// if you just have enough or too many, grab enough for the numberOfReps
    	if ( (numberOfSeeds == numberOfReps) || (numberOfSeeds > numberOfReps ) ) {
    		for (int i=0; i < numberOfReps; i++ ) {
    			Element seed = seeds.get(i);
    			randomSeeds.add(seed.getTextTrim());
    		}

    		return;
    	}
    	
    	if ( numberOfSeeds < numberOfReps ) {
    		// get all that you have in the study.xml file
    		for (int i=0; i < numberOfSeeds; i++ ) {
    			Element seed = seeds.get(i);
    			randomSeeds.add(seed.getTextTrim());
    		}   		
    		// and add some more
    		addSeeds();
    		
    	}
       	
    }
    
    private void addSeeds() {
    	int numberOfSeeds = randomSeeds.size();
    	String aSeed = "2558775454";
        long seed = new Long(aSeed).longValue();
        random.setSeed(seed);
        for (int i = numberOfSeeds; i < numberOfReps; i++) {
        	randomSeeds.add(i, Long.toString(random.nextLong()));
        }
    }
	
	public void writeSeeds() {
		String newline = System.getProperty("line.separator");
        try {
            FileWriter fw = new FileWriter(new File("seeds.txt"));
            for (int i=0; i<numberOfReps;i++ ) {
            	fw.write(randomSeeds.get(i));
            	fw.write(newline);
            }
            
            fw.close();
        } catch (IOException ioe) {
            System.out.println("Error attempting to open: seeds.txt");
            ioe.printStackTrace(System.out);
            System.exit(-1);
        }
	}

	
	public void runPythagoras(int repNum) {
		addCommandLineParameter(scenarioFileName);
    	addCommandLineParameter(randomSeeds.get(repNum));
    	addCommandLineParameter("1");
    	addCommandLineParameter("false");
    	String rep = Integer.toString(repNum + 1);
    	addCommandLineParameter(moeFileName + "." + rep);
    	try {
    		
    		loader = new MyURLClassLoader(urls);
    		Class<?> pyth = loader.findClass("com.prototype.terrain.Pyth");
 
    		String[] args = cmdlineToStringArray();
       		Method main = pyth.getMethod("main", args.getClass());
    		main.invoke(null, (Object)args);
    		
    	}

    	catch (NoSuchMethodException cnf) {
    		System.out.println("NoSuchMethodException: com.prototype.terrain.Pyth");
    		System.exit(-1);
    	}
    	catch (InvocationTargetException cnf) {
    		System.out.println("IllegalAccessException: com.prototype.terrain.Pyth");
    		System.exit(-1);
    	}
    	catch (IllegalAccessException cnf) {
    		System.out.println("IllegalAccessException:  com.prototype.terrain.Pyth");
    		System.exit(-1);
    	}
    	
	}
	
	private String[] cmdlineToStringArray() {
		int size = cmdarray.size();
		String[] result = new String[size];
		int i=0;
		for (String cmd : cmdarray ) {
			result[i++] = cmd;
		}
		return result;

	}
	

    
    public void addCommandLineParameter(String s) {
    	cmdarray.add(s);
    }
    
    
    public void setStudyXMLFileName(String s) {
    	studyXMLFileName = s;
    }
    
    public void setScenarioFileName(String s) {
    	scenarioFileName = s;
    }
    
    public void setNumberOfReplications(String n) {
    	numberOfReps = Integer.parseInt(n);
    }
    
    public void setMOEFileName(String m) {
    	moeFileName = m;	
    }
    
    public static void main(String[] args) {
    	RunPythagoras pyth = new RunPythagoras();
    	pyth.setStudyXMLFileName(args[0]);
    	pyth.setScenarioFileName(args[1]);
    	pyth.setMOEFileName(args[2]);
    	pyth.makeRuns();
    }
}

class MyURLClassLoader extends URLClassLoader {
	URL[] urls; 
	
	public MyURLClassLoader(URL[] urls) {
		super(urls);
		this.urls = urls;
	}
	
	public Class<?> findClass(String className) {
		Class<?> c = null;
		try {
			c = super.findClass(className);
		}
		catch(ClassNotFoundException cnf) {
			System.out.println("Could not find class: " + className);
			for (URL url : urls) {
				System.out.println("URL used: " + url.toString());	
			}

			System.exit(-1);
		}
		
		return c;
		
	}
	
}
