/**
 * 
 */
package oldmcdata.model.pythagoras;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;


/**
 * Runs Pythagoras model on DoD HPC resources, primarily Linux HPC clusters.
 * 
 * @author Stephen C. Upton
 * @version 1.0, 16 Oct 2008
 *
 */
public class RunPythagorasOnHPC {
	ArrayList<String> cmdarray;
	ArrayList<String> randomSeeds;
	String studyXMLFileName, scenarioFileName, moeFileName;
	int numberOfReps = 1;
	String pathSeparator, javaHome, pythJarToUse;
    private Random random = new Random();
    private boolean timeSeriesDesired = false;
    private long waitForFilesTime = 60000L;
    private long totalJobTime = 120000L;
    private long currentTime = 0L;
	
	public RunPythagorasOnHPC() {
		cmdarray = new ArrayList<String>();
		randomSeeds = new ArrayList<String>();
	}	
	
	
	public void makeRuns() {
		pathSeparator = System.getProperty("path.separator");
		System.out.println("using java version: " + System.getProperty("java.version"));
		openStudyXMLFile();
		for (int i = 0; i< numberOfReps; i++) {
			runPythagoras(i);

			runCommandLine();
			System.out.println("completed run: " + i);
			cmdarray.clear();
		}

		System.out.println("completed submission of runs");
		
		watchFiles();
			
	}

	@SuppressWarnings("unchecked")
	public void openStudyXMLFile() {
		try {
			SAXReader xmlReader = new SAXReader();
			Document studyDoc = xmlReader.read(studyXMLFileName);
			Element rootElement = studyDoc.getRootElement();
			Element modelParameters = rootElement.element("ModelParameters");
			String numberOfRepsString = modelParameters.elementText("NumberReplicates");
			setNumberOfReplications(numberOfRepsString);
			List<Element> seeds = modelParameters.elements("InitialRandomSeed");
			setSeeds(seeds);

		} 

		catch (DocumentException de) {
			System.out.println("Error attempting to parse: " + studyXMLFileName);
			de.printStackTrace(System.out);
			System.exit(-1);
		}
	}
	
	private void setSeeds(List<Element> seeds) {
		int numberOfSeeds = 0;
		if ( seeds == null ) {
			addSeeds();
			return;
		}

		numberOfSeeds = seeds.size();
		// if you just have enough or too many, grab enough for the numberOfReps
		if ( (numberOfSeeds == numberOfReps) || (numberOfSeeds > numberOfReps ) ) {
			for (int i=0; i < numberOfReps; i++ ) {
				Element seed = seeds.get(i);
				randomSeeds.add(seed.getTextTrim());
			}

			return;
		}

		if ( numberOfSeeds < numberOfReps ) {
			// get all that you have in the study.xml file
			for (int i=0; i < numberOfSeeds; i++ ) {
				Element seed = seeds.get(i);
				randomSeeds.add(seed.getTextTrim());
			}   		
			// and add some more
			addSeeds();

		}

	}

	private void addSeeds() {
		int numberOfSeeds = randomSeeds.size();
		String aSeed = "2558775454";
		long seed = new Long(aSeed).longValue();
		random.setSeed(seed);
		for (int i = numberOfSeeds; i < numberOfReps; i++) {
			randomSeeds.add(i, Long.toString(random.nextLong()));
		}
	}
	
	public void runPythagoras(int repNum) {
		addCommandLineParameter(javaHome);
    	addCommandLineParameter("-jar");
    	addCommandLineParameter(pythJarToUse);
    	if ( timeSeriesDesired ) {
    		addCommandLineParameter("-L");
    	}
		addCommandLineParameter(scenarioFileName);
    	addCommandLineParameter(randomSeeds.get(repNum));
    	addCommandLineParameter("1");
    	addCommandLineParameter("false");
    	addCommandLineParameter(moeFileName + "." + repNum);
//		Pyth.main(cmdToArgs());
	}
	
	public String[] cmdToArgs() {
		String[] args = new String[cmdarray.size()];
		int numargs = 0;
		for (String cmd : cmdarray) {
			args[numargs++] = cmd;
 		}
		
		return args;
	}
	
    public void runCommandLine() {
        System.out.println("Command Line: ");
        for (String cmd : cmdarray ) 
            System.out.print(cmd+ " ");
        System.out.println();
   		
        try {
        	Process pp = new ProcessBuilder(cmdarray).start();
        	BufferedReader results = new BufferedReader(
        			new InputStreamReader(pp.getInputStream()));
        	String s;
        	while ((s = results.readLine()) != null)
        		System.out.println(s);
        	BufferedReader errors = new BufferedReader(
        			new InputStreamReader(pp.getErrorStream()));
        	while ((s = errors.readLine()) != null ) {
        		System.err.println(s);
        	}
        	pp.waitFor();        	

        } catch (Exception ex) {
            ex.printStackTrace(System.out);
            System.exit(-1);
        }


    }
    
    public void watchFiles() {
    	while (totalJobTime < currentTime ) {
    		try {
    			System.out.println("Watching Files......");
    			Thread.sleep(waitForFilesTime);	
    			currentTime += waitForFilesTime;
    		}
    		catch (InterruptedException ie) {
    			System.err.println("Error in waiting for files... terminating program.");
    			ie.printStackTrace(System.err);
    			System.exit(-1);
    		}
    	}

    	
    }
    
    public void addCommandLineParameter(String s) {
    	cmdarray.add(s);
    }
	
    public void setJavaLocation(String javaLocation) {
    	javaHome = javaLocation;
    }
    
    public void setPythJarToUse(String pythJarToUse) {
    	this.pythJarToUse = pythJarToUse;
    }
    
    public void setStudyXMLFileName(String s) {
    	studyXMLFileName = s;
    }
    
    public void setScenarioFileName(String s) {
    	scenarioFileName = s;
    }
    
    public void setNumberOfReplications(String n) {
    	numberOfReps = Integer.parseInt(n);
    }
    
    public void setMOEFileName(String m) {
    	moeFileName = m;	
    }
    
    public void setTimeSeries(String ts) {
    	if ( ts.equalsIgnoreCase("-L")) {
    		timeSeriesDesired = true;
    	}
    }
	
    public static void main(String[] args) {
    	RunPythagorasOnHPC pyth = new RunPythagorasOnHPC();
    	pyth.setJavaLocation(args[0]);
    	pyth.setPythJarToUse(args[1]);
    	pyth.setStudyXMLFileName(args[2]);
    	pyth.setScenarioFileName(args[3]);
    	pyth.setMOEFileName(args[4]);
    	if ( args.length == 6 ) {
    		pyth.setTimeSeries(args[5]);
    	}
    	pyth.makeRuns();
    }
}
