/*
 * Created on May 6, 2004
 *
 */
package oldmcdata.postprocessors;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;



/**
 *Copyright: 2000 - 2005
 *   Referentia Systems, Inc.
 * @author Stephen C. Upton
 *
 */
public class DataCollector {
	private final static int APPENDBYROW = 0,
							APPENDBYCOLUMN = 1;
	private int fileCount = 0; // used if pattern fails to find replicate number
	private List allData, currentData, finalData;
	private boolean useColumnNames, useRowNames, isExtraInfo = false;
	private Map myMap;
	private PrintWriter logFileWriter;
	private Pattern replicatePattern;
	private String replicateNumber = null, extraInfo = null, extraInfoHeader;
	private String currentFileName;
	private int appendBy = APPENDBYROW;
	private String delimiter = ",";
	private PostProcessTask myTask;
	
	public DataCollector(PostProcessTask task){
		myTask = task;
		this.configure();
	}
	
	public void configure() {
		allData = new ArrayList();
		this.logFileWriter = myTask.getLogFileWriter();
		this.useColumnNames = myTask.getUseColumnNames();
		this.useRowNames = myTask.getUseRowNames();
		this.myMap = myTask.getCollectorMap(); 
		String appendByString = (String)myMap.get(PostProcessTask.APPENDBY); 
		if ( appendByString.equals(PostProcessTask.COLUMN) )
			appendBy = APPENDBYCOLUMN;
		else
			appendBy = APPENDBYROW;		
		this.setReplicatePattern(myTask.getReplicatePattern());
	}
	
	public void setCurrentFileName(String fileName) {
		currentFileName = fileName;	
	}
	
	
	public void setReplicatePattern(Pattern p) {
		replicatePattern = p;
	}	
	
	public void globData(List selectedData) {
		fileCount++;
		currentData = selectedData;
		determineReplicateNumber();
		determineExtraInfo();


		switch( appendBy ) {
			case APPENDBYROW: appendByRow(); break;
			case APPENDBYCOLUMN: appendByColumn(); break;
			default: appendByRow();
		}

		if ( useColumnNames )
			logFileWriter.println("Used column names");
		else
			logFileWriter.println("Did not use column names");
		if ( useRowNames )
			logFileWriter.println("Used row names");
		else
			logFileWriter.println("Did not use row names");


	}
	
	public List getCollectedData() {
		finalData = new ArrayList();
		switch( appendBy ) {
			case APPENDBYROW: concatenateByRow(); break;
			case APPENDBYCOLUMN: concatenateByColumn(); break;
		}
		return finalData;
	}
	
	private void concatenateByRow() {
		constructHeader();

		for (int i=0; i < allData.size();i++ ) {
			List dataline = (List)allData.get(i);
			finalData.add(convertListToString(dataline));
		}
		
	}
	
	private void constructHeader() {
		if ( myTask.getUseHeaderRow() == false ) 
			return;
		String headerRow = (String)myTask.getHeaderRows().get(0);
		String[] header = headerRow.split(delimiter);
		List headerList = new ArrayList();
		Set columnNumbers = myTask.getColumnNumbers();
		// add header text for replicate
		headerList.add("Rep");
		// add header text for extra info, if needed
		if ( isExtraInfo )
		    headerList.add(extraInfoHeader);
		for (Iterator columnIterator = columnNumbers.iterator();columnIterator.hasNext();) {
			int columnToSelect = ((Integer)columnIterator.next()).intValue();
			headerList.add(header[columnToSelect-1]);
		}
		
		finalData.add(convertListToString(headerList));
	}
	
	private void concatenateByColumn() {
	    int maxNumberOfRows = 0;
		int numberOfColumns = allData.size();
		int[] subColumns = new int[numberOfColumns];
	    //determine maximum number of rows across all columns
	    for (int j=0;j<numberOfColumns;j++) {
			List column = (List)allData.get(j);
			List firstRow = (List)column.get(0);
			subColumns[j] = firstRow.size();
			int numberOfRows = column.size();
			maxNumberOfRows = Math.max(maxNumberOfRows, numberOfRows);
	    }


		for (int i=0;i<maxNumberOfRows;i++) {
			List rowString = new ArrayList();
			for (int j=0;j<numberOfColumns;j++) {
				List columnData = (List)allData.get(j);
				int rowSize = columnData.size();
				if ( i < rowSize ) {
					List rowData = (List)columnData.get(i);
					rowString.add(convertListToString(rowData));
				}
				else {
				    rowString.add(numberOfBlankStrings(subColumns[j]));  
				}

			}
			finalData.add(convertListToString(rowString));
			
		}
	}
	
	private String numberOfBlankStrings(int b) {
	    StringBuffer sb = new StringBuffer();
	    for (int i=0;i<b;i++){
	        sb.append(" ");
	        sb.append(delimiter);
	    }
		// remove the last delimiter
		sb.deleteCharAt(sb.length() - 1);
		
	    return sb.toString();
	}

	
	private void determineReplicateNumber() {
		String result = null;
		replicateNumber = Integer.toString(fileCount);

		Matcher matcher = replicatePattern.matcher(currentFileName);
		if ( matcher.find() ) 
			result = matcher.group(1);
		
		if ( result == null ) {
			logFileWriter.println("Could not determine replicate number from pattern: " +
									replicatePattern.pattern());
			logFileWriter.println("Using file count as replicate number: " + replicateNumber);
		}
		else {
			replicateNumber = result.trim();
			logFileWriter.println("Using replicate number: " + replicateNumber);
			logFileWriter.println("Determined from file: " + currentFileName);
		}
						
	}
	
	private void determineExtraInfo() {
		String result = null;
		Map extraInfoMap = myTask.getDataMap(PostProcessTask.EXTRAINFO);
		isExtraInfo = !extraInfoMap.isEmpty();
		
		if ( !isExtraInfo ) return;
		extraInfoHeader = (String)extraInfoMap.get(PostProcessTask.HEADERNAME);
		
		Pattern extraInfoPattern = (Pattern)extraInfoMap.get(PostProcessTask.PATTERN);
		String extraInfoValue = (String)extraInfoMap.get(PostProcessTask.VALUE);
		if ( extraInfoValue == null ) return;
		Matcher matcher = extraInfoPattern.matcher(extraInfoValue);
		if ( matcher.find() ) 
			result = matcher.group(1);
		
		if ( result == null ) {
			logFileWriter.println("Could not determine extra info from pattern: " +
			        extraInfoPattern.pattern());
			isExtraInfo = false;
		}
		else {
		    extraInfo = result.trim();
			logFileWriter.println("Added extra info: " + extraInfo);
		}
	}
	
	private void appendByRow() {
		logFileWriter.println("Appending replicate " + replicateNumber + " data by row");
		for (int i=0; i < currentData.size();i++ ) {
			List dataline = (List)currentData.get(i);
			dataline.add(0,replicateNumber);
			if ( isExtraInfo )
			    dataline.add(1,extraInfo);
			allData.add(dataline);
		}

	}

	
	private void appendByColumn() {
		logFileWriter.println("Appending replicate " + replicateNumber + " data by column");
		List replicateColumn = new ArrayList();
		if ( useColumnNames ) {
			replicateColumn.add(makeReplicateRow());
			makeColumn(replicateColumn, 0);

		} //not using column names
		else {
			replicateColumn.add(makeReplicateRow());
			makeColumn(replicateColumn, 1);
		}	
		if ( useRowNames ) 
			if ( fileCount != 1) {
				for (int i=0;i<replicateColumn.size();i++) {
					List row = (List)replicateColumn.get(i);
					row.remove(0);
				}
			}
		
		allData.add(replicateColumn);
	}
	
	private List makeReplicateRow() {
		List replicateRow = new ArrayList(); //list of Strings equal to replicateNumber
		List headerColumn = (List)currentData.get(0);
		for (int i=0;i<headerColumn.size();i++) {
			replicateRow.add(replicateNumber);
		}
		
		return replicateRow;
	}
	
	private void makeColumn(List column, int startIndex) {
		for (int i=startIndex;i<currentData.size();i++) {
			List dataline = (List)currentData.get(i);
			column.add(dataline);
		}
		
	}
	
	
	/* Convenience function to convert the list of strings to a single string.
	 * 
	 */
	private String convertListToString(List listOfStrings) {
		StringBuffer sb = new StringBuffer();
		if ( listOfStrings.size() == 0 )
			return sb.toString(); // empty string		
		for(Iterator stringList = listOfStrings.iterator(); stringList.hasNext(); ) {
			sb.append((String)stringList.next());
			sb.append(delimiter);
		}

		// remove the last delimiter
		sb.deleteCharAt(sb.length() - 1);
		
		return sb.toString();
	}
}
