/*
 * Created on May 5, 2004
 *
 */
package oldmcdata.postprocessors;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;


/**
 *Copyright: 2000 - 2005
 *   Referentia Systems, Inc.
 * @author Stephen C. Upton
 *
 */
public class DataSelector {
	private PrintWriter logFileWriter;
	private List selectedData;
	private String delimiter = ",";
	private Set columnNumbers;
	private RowSelector rowSelector;
	private ColumnSelector columnSelector;
	private PostProcessTask myTask;
	
	public DataSelector(PostProcessTask task) {
		myTask = task;
		logFileWriter = myTask.getLogFileWriter();
		columnNumbers = new TreeSet();
		rowSelector = new RowSelector(myTask, PostProcessTask.SELECTROWS);
		columnSelector = new ColumnSelector(myTask);
	}
	
	public Set determineColumns() {
		if ( myTask.getUseHeaderRow() == true ) {
			List headerRows = myTask.getHeaderRows();
			//use the header row from the first file
			columnNumbers = columnSelector.determineColumns((String)headerRows.get(0));
		}
		logFileWriter.println("Column numbers selected: " + columnNumbers.toString());
		return columnNumbers;
	}

	public List processData(List data) {
		selectedData = new ArrayList();
		// if there are no column numbers, then leave data alone, i.e, all data is selected
		// however, we need to split the data since data is a List of Strings and we
		// need a List of List of Strings returned
		if ( columnNumbers.size() == 0 ) {
			for (Iterator dataLines = data.iterator(); dataLines.hasNext(); ) {
				String dataLine = (String)dataLines.next();
				String[] dataRow = dataLine.split(delimiter);
				List rowData = new ArrayList();
				for (int i=0;i<dataRow.length;i++) {
					rowData.add(dataRow[i]);
				}
				selectedData.add(rowData);
			}
			return selectedData;
		}


		boolean selectAllRows = rowSelector.selectAllRows();	
		for (Iterator dataLines = data.iterator(); dataLines.hasNext(); ) {
			String dataLine = (String)dataLines.next();
			if ( selectAllRows || rowSelector.lineMatches(dataLine) ) {
				String[] dataRow = dataLine.split(delimiter);
				List columnData = new ArrayList();
				for (Iterator columnIterator = columnNumbers.iterator();columnIterator.hasNext();) {
					int columnToSelect = ((Integer)columnIterator.next()).intValue();
					if ( columnToSelect > dataRow.length )
						columnData.add("NA");
					columnData.add(dataRow[columnToSelect-1]);
				}
				selectedData.add(columnData);
			}


			
		}
		// selectedData is a List of Lists of Strings, where the ith entry of the first
		// list is the ith row selected, and the ith entry of that is the column entry
		return selectedData;

	}

}
