/*
 * Created on May 1, 2004
 *
 */
package oldmcdata.postprocessors;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;

import org.dom4j.Element;


/**
 *Copyright: 2000 - 2005
 *   Referentia Systems, Inc.
 * @author Stephen C. Upton
 *
 */
public class PostProcessTask {
	public final static String 
		TASKNAME = "TaskName",
		FILEDIRECTORY = "FileDirectory",
		FILEPATTERN = "FilePattern", 
		INFILE = "InFile", 
		OUTFILE = "OutFile", 
		EXTRAINFO = "ExtraInfo",
		SKIPLINES = "SkipLines",
		SELECTCOLUMNS = "SelectColumns",
		SELECTROWS = "SelectRows",
		STARTSWITH = "StartsWith", 
		CONTAINS = "Contains", 
		NUMBER = "Number",
		PATTERN = "Pattern", 
		HEADERNAME = "HeaderName",
		HEADERROW = "UseHeaderRow",
		COLUMNNAMES = "UseColumnNames", 
		ROWNAMES = "UseRowNames",
		PREPEND = "Prepend", 
		APPENDBY = "AppendBy", 
		FILENAME = "FileName", 
		VALUE = "Value",
		COLUMN = "Column",
		ROW = "Row",
		CURRENTFILE = "CurrentFile",
		REPLICATEPATTERN = "ReplicatePattern";	
	
	private static int taskNumber = 0;
	private PrintWriter logFileWriter;
	private List fileList,data, selectedData,headerRows;
	private Map taskData, skipLinesMap, selectColumnsMap, selectRowsMap,
							collectorMap,extraInfoMap;
	private Set columnNumbers;
	private File currentFile;
	private Element taskElement;
	private Pattern filePattern, replicatePattern;
	private String taskName,fileDirectory,outputFileName = null;
	private boolean useColumnNames = true, useRowNames = false, useHeaderRow=true;
	private DataReader reader;
	private DataSelector selector;
	private DataCollector collector;
	private FileMatcher fileMatcher;
	
	public PostProcessTask(Element task, PrintWriter logFileWriter) {
		this.taskElement = task;
		skipLinesMap = new HashMap();
		selectColumnsMap = new HashMap();
		selectRowsMap = new HashMap();
		collectorMap = new HashMap();
		extraInfoMap = new HashMap();
		taskData = new HashMap();
		headerRows = new ArrayList();
		taskData.put(EXTRAINFO, extraInfoMap);
		taskData.put(SKIPLINES, skipLinesMap);
		taskData.put(SELECTCOLUMNS, selectColumnsMap);
		taskData.put(SELECTROWS, selectRowsMap);
		this.logFileWriter = logFileWriter;	
		taskNumber++;
	}
	
	public Element getTaskElement() {
		return taskElement;
	}
	
	public PrintWriter getLogFileWriter() {
		return logFileWriter;
	}
	
	public Map getDataMap(String mapValue) {
		return (Map)taskData.get(mapValue);
	}
	
	public Map getCollectorMap() {
		return collectorMap;
	}
	
	public int getTaskNumber() {
		return taskNumber;
	}
	
	public String getTaskName() {
		return taskName;
	}

	public void setFilePattern(Pattern pat) {
		filePattern = pat;
	}
	
	public Pattern getFilePattern() {
		return filePattern;
	}
	
	public void setReplicatePattern(Pattern pat) {
		replicatePattern = pat;
	}
	
	public Pattern getReplicatePattern() {
		return replicatePattern;
	}
	
	public void setUseColumnNames(boolean b) {
		useColumnNames = b;
	}
	
	public boolean getUseColumnNames() {
		return useColumnNames;
	}
	
	public void setUseRowNames(boolean b) {
		useRowNames = b;
	}
	
	public boolean getUseRowNames() {
		return useRowNames;
	}
	
	public void setUseHeaderRow(boolean b) {
		useHeaderRow = b;
	}
	
	public boolean getUseHeaderRow() {
		return useHeaderRow;
	}
	
	public void setTaskName(String name) {
		taskName = name;
	}
	
	public void setFileDirectory(String dir) {
		fileDirectory = dir;
	}
	
	public String getFileDirectory() {
		return fileDirectory;
	}
	
	public List getHeaderRows() {
		return headerRows;
	}
	
	public Set getColumnNumbers() {
		return columnNumbers;
	}
	
	public void doTask() {
		PostProcessTaskValidator validator = new PostProcessTaskValidator(this);
		// if the task element isn't sufficiently defined, then leave
		if ( validator.validate() == false ) return;

		// intialize the fileMatcher and look for matching files for this Task
		fileMatcher = new FileMatcher(this);
		fileMatcher.matchFiles();
		// if there are no matching files to process then we're done!
		if ( !getFilesToMatch() ) return;
				
		reader = new DataReader(this);
		selector = new DataSelector(this);
		collector = new DataCollector(this);
		// first, read in the data except for those rows skipped defined by skip lines,
		// in essence, selecting the rows you want to process
		// second, select the data specified by SelectColumns
		// finally, collect the data from each of the previous select data operations
		int filecount = 1;
		for (Iterator files = fileList.iterator();files.hasNext();) {
			currentFile = (File)files.next();
			if ( !readData(currentFile) ) continue;
			
//			The result from the selector is a List of Strings, where each String is a row of 
//			data for that file, and the length of that String is the columns selected.
			if ( filecount == 1 )
				columnNumbers = selector.determineColumns();
			selectedData = selector.processData(data);
			collectData();
			filecount++;
		}

		writeData();
		
	}

	
	private boolean getFilesToMatch() {
		fileList = fileMatcher.getMatchingFiles();

		if ( fileList.size() == 0 ) 
			return false;			
		
		return true;
	}
	
	/* Read in data, skipping rows defined by the skip lines element of the postprocessor 
	 * XML directive file.
	 * 
	 */
	private boolean readData(File fileToProcess) {
		boolean result = true;
		
		data = reader.getData(fileToProcess);
		if ( data.size() == 0 )
			result = false;
		
		return result;

	}

	
	private void collectData() {
		collector.setCurrentFileName(currentFile.getName());		
		collector.globData(selectedData);
	}
	
	private void writeData() {
		List collectedData = collector.getCollectedData();
		if ( collectedData.size() == 0 ) {
			logFileWriter.println("NOTICE: No data to output");
			return;
		}		
		outputFileName = (String)collectorMap.get(FILENAME);
		File outputFile = new File(outputFileName);
		try {
			PrintWriter outFileWriter = new PrintWriter(new FileWriter(outputFile));
			for (Iterator dataItems = collectedData.iterator(); dataItems.hasNext(); ) {
				String lineOfData = (String)dataItems.next();
				outFileWriter.println(lineOfData);
			}
			outFileWriter.close();
			logFileWriter.println("Completed writing data to: " + outputFile.getName());
			logFileWriter.println("**************************");
			logFileWriter.println();
		} catch (IOException ioe) {
			System.out.println("Error attempting to open: " + outputFile.getName());
			ioe.printStackTrace(System.out);
			logFileWriter.close();
			System.exit(-1);
		}			
	}


}

