/**
 * 
 */
package oldmcdata.postprocessors.model;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

/**
 * @author Stephen C. Upton
 * @date 3 December 2007
 *
 */
public class PythagorasMOEHeader {
	private File scenarioFile;
	private String scenarioDir, outFile;
	private Element moeOutput;
	private List<String> moeHeader;
	private String delimiter = ",";
	
	public PythagorasMOEHeader(String dir, String scenarioFileName) {
		scenarioFile = new File(dir, scenarioFileName);
		scenarioDir = dir;
        moeHeader = new ArrayList<String>();
    }
	
	public PythagorasMOEHeader(String scenarioFileName) {
		scenarioFile = new File(scenarioFileName);
		scenarioDir = scenarioFile.getParent();
		moeHeader = new ArrayList<String>();
	}
	
	@SuppressWarnings("unchecked")
    private void composeMOENames() {
		openScenarioFile();
		List<Element> moeTemplates = moeOutput.elements("moeTemplate");
			for (Element moeTemplate : moeTemplates ) {
				List<Element> fieldNames = moeTemplate.elements("fieldName");
				String agentName = moeTemplate.attributeValue("name");
				for (Element fieldName : fieldNames ) {
					String moeName = fieldName.elementTextTrim("label");
					String fullMoeName = agentName + ":" + moeName;
					moeHeader.add(fullMoeName);
					System.out.println("MoeName: " + fullMoeName);	
				}

			}
    }
	
	public void setOutputFile(String outFileName) {
		outFile =  outFileName;
	}
    
    public void setDelimiter(String d) {
    	delimiter = d;
    }
    
    public void setScenarioFileName(String scenarioFileName) {
    	scenarioFile = new File(scenarioFileName);
    }
    
    public void setScenarioDirectory(String dir) {
    	
    }
    
	public void openScenarioFile() {
		try {
			SAXReader xmlReader = new SAXReader();
			Document basecaseDoc = xmlReader.read(scenarioFile);
			Element rootElement = basecaseDoc.getRootElement();
			moeOutput = rootElement.element("moeOutput");

		} 
//		catch (MalformedURLException mue) {
//			System.out.println("Can not form URL from this file: " + basecase);
//			mue.printStackTrace(System.out);
//			System.exit(-1);
//		} 
		catch (DocumentException de) {
			System.out.println("Error attempting to parse: " + scenarioFile);
			de.printStackTrace(System.out);
			System.exit(-1);
		}

	}
	
	public String getHeader() {
		return getHeader(delimiter);
	}
	
	public String getHeader(String d) {
		setDelimiter(d);
		composeMOENames();
		StringBuilder header = new StringBuilder();
		for (String moeName : moeHeader ) {
			header.append(moeName);
			header.append(delimiter);
		}
		header.deleteCharAt(header.length() - 1 );
		
		return header.toString();
	}
	
	public void writeHeader() {
        try {
            FileWriter fw = new FileWriter(new File(scenarioDir, outFile));
            fw.write(getHeader());
            fw.close();
        } catch (IOException ioe) {
            System.out.println("Error attempting to open: " + outFile.toString());
            ioe.printStackTrace(System.out);
            System.exit(-1);
        }
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		String basecaseFileName = args[0];
		String outFileName = args[1];
		String dir = System.getProperty("user.dir");
		PythagorasMOEHeader header = new PythagorasMOEHeader(dir, basecaseFileName);
		header.setOutputFile(outFileName);
		header.writeHeader();

	}

}
