package oldmcdata.postprocessors.model;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import oldmcdata.analyzers.utils.OutputDataFileHandler;
import oldmcdata.utils.DefaultStudyXMLHandler;
import oldmcdata.utils.StudyXML;
import oldmcdata.utils.StudyXMLHandler;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2007
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class PythagorasPostProcessor {
    protected BufferedReader fileToProcess;
    protected StudyXMLHandler studyXMLHandler;
    private String studyDirName;
    private FileWriter logFileWriter;
    private String headerLine,  delimiter = ",", lineSeparator,
    			prependIdentifier, repSeparator = ".";
    private int numberOfReplicates;
    private List  iterationElements, excursions;
    private List<String> dataList, moeFileNames;
    private File dataFile, logFile;
    private boolean doPrepend = false, dofitness = false;
    private boolean multipleRepsPerFile = false;

    private File basecase;
	
	
	public PythagorasPostProcessor(String studyDirName, String studyFileName, String dataFileName) {
        studyXMLHandler = new DefaultStudyXMLHandler(studyDirName, studyFileName);
        this.studyDirName = studyDirName;
        dataFile = new File(studyXMLHandler.getStudyDirectory(), dataFileName);
        logFile = new File(studyXMLHandler.getStudyDirectory(), "PythagorasPostProcessor.log");
        lineSeparator = System.getProperty("line.separator");
        openLogFile();
    }

    public void setPrependIdentifier(String prepend) {
        prependIdentifier = prepend;
        doPrepend = true;
    }
    
    public void setReplicationSeparator(String sep) {
    	repSeparator = sep;
    }
    
    public void openLogFile() {
        try {
        	logFileWriter = new FileWriter(logFile.toString(), true);
        } catch (IOException ioe) {
            System.err.println("Error attempting to open log file: " + logFile.toString());
            System.exit(-1);
        }
    }
    
    public void setMultipleRepsPerFile(boolean t) {
    	multipleRepsPerFile = t;
    }

	@SuppressWarnings("unchecked")
    public void postProcess() {
        studyXMLHandler.openStudyXMLDocument();
        numberOfReplicates = Integer.parseInt(studyXMLHandler.getNumberReplicates());
        dataList = new ArrayList<String>();
        constructHeader();
        iterationElements = studyXMLHandler.getIterations();
        if ( iterationElements != null && iterationElements.size() != 0) {
        	System.out.println("doing iterations from study.xml format");
        	for (int i = 0; i < iterationElements.size(); i++) {
        		Element iterationElement = (Element) iterationElements.get(i);
        		processIteration(iterationElement);
        		// print iteration at a time, assuming all of the data would not fit into memory
        	}
        }
        else {
        	//do the excursions; assumes we are now using MHPCC study.xml format (no Iteration element)
        	Element modelRunInformationElement = studyXMLHandler.getModelRunInformationElement();
        	System.out.println("doing just excursions from MHPCC study.xml format");
        	excursions = modelRunInformationElement.elements("Excursion");
        	moeFileNames = getMOEFileNames(modelRunInformationElement);
            processExcursions();
            printData();
        }

        try {
        	logFileWriter.close();
        } catch (IOException ioe) {
            System.err.println("Error attempting to close log file: " + logFile.toString());
            System.exit(-1);
        }        
    }
	
	@SuppressWarnings("unchecked")
    public List<String> getMOEFileNames(Element modelRunInformationElement) {
        List<String> moeFileNames = new ArrayList<String>();
        int numberOfExcursions = modelRunInformationElement.elements(StudyXML.EXCURSION).size();
        Element excursionFileInfo = modelRunInformationElement.element(StudyXML.EXCURSIONFILEINFO);
        String moeDirName = excursionFileInfo.element(StudyXML.MOEDIR).getTextTrim();
        File moeDir = new File(studyDirName, moeDirName);
        String moeFileStub = excursionFileInfo.element(StudyXML.MOEFILESTUB).getTextTrim();
        for (int i = 0; i < numberOfExcursions; i++) {
            String moeFileName = moeFileStub + Integer.toString(i);
            File moeFile = new File(moeDir, moeFileName);
            moeFileNames.add(moeFile.toString());
        }
        return moeFileNames;
    }

	@SuppressWarnings("unchecked")
    private void processIteration(Element iterationElement) {
        excursions = studyXMLHandler.getExcursions(iterationElement);
        moeFileNames = studyXMLHandler.getMOEFileNames(iterationElement);
        processExcursions();
        printData();
    }

    private void processExcursions() {
        for (int j = 0; j < excursions.size(); j++) {
            Element excursion = (Element) excursions.get(j);
            String moeFileName = (String) moeFileNames.get(j);
            processExcursion(excursion, moeFileName, j);
        }
    }

	@SuppressWarnings("unchecked")
    private void processExcursion(Element excursion, String moeFileName, int whichone) {
        StringBuilder sb = new StringBuilder();
        List<String> excursionValues = new ArrayList<String>();
        String excursionID = excursion.attributeValue(StudyXML.ID);
        if ( excursionID == null )
        	excursionID = Integer.toString(whichone);
        String iterationID = excursion.getParent().attributeValue(StudyXML.ID);
		if ( iterationID != null ){
			sb = new StringBuilder("I").append(iterationID);
		}

        sb.append("E").append(excursionID);
        if (doPrepend)
            sb.insert(0, prependIdentifier);
        excursionValues.add(sb.toString());
        excursionValues.addAll(studyXMLHandler.getExcursionValues(excursion));
        if (dofitness) {
            String fitness = excursion.element(StudyXML.FITNESS).getTextTrim();
            excursionValues.add(fitness);
        }
        String values = composeValuesAsOneString(excursionValues);
        if (multipleRepsPerFile)
        	processMultipleReplicatesPerMOEFile(moeFileName, values);
        else
        	processSingleReplicatePerMOEFile(moeFileName, values);
    }

    private String composeValuesAsOneString(List<String> values) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < values.size(); i++) {
            String vName = (String) values.get(i);
            sb.append(vName).append(delimiter);
        }
        sb.deleteCharAt(sb.length()-1);
        return sb.toString();
    }

    public BufferedReader openFile(String fileName) {
    	BufferedReader dataReader = null;
        try {
            dataReader = new BufferedReader(
                    new FileReader(fileName));
        } catch (IOException e) {
            System.err.println("Can not find or open file: " + fileName);
        }
        return dataReader;
    }
    
    protected void processSingleReplicatePerMOEFile(String moeFileName, String prependString) {
        try {
        	String noOutput = "NoOutput";
            for (int i = 1; i <= numberOfReplicates; i++) {
            	String repMOEFileName = moeFileName + repSeparator + i;
            	fileToProcess = openFile(repMOEFileName);
            	if ( fileToProcess == null ) {
            		logFileWriter.write("Can not find MOE file: " + repMOEFileName);
            		dataList.add(prependString + delimiter + noOutput);
            		logFileWriter.write(lineSeparator);
            		continue;
            	}
                String dataLine = fileToProcess.readLine();
                if ( dataLine == null )
                	dataLine = noOutput;
                dataList.add(prependString + delimiter + dataLine);
                fileToProcess.close();
            }
            
        } catch (IOException e) {
            e.printStackTrace(System.err);
        }
    }

    protected void processMultipleReplicatesPerMOEFile(String moeFileName, String prependString) {
    	try {
    		fileToProcess = openFile(moeFileName);
    		if ( fileToProcess == null ) {
    			logFileWriter.write("Can not find MOE file: " + moeFileName);
    			logFileWriter.write(lineSeparator);
    			return;
    		}
    		for (int i = 0; i < numberOfReplicates; i++) {
    			String dataLine = fileToProcess.readLine();
    			dataList.add(prependString + delimiter + dataLine);
    		}
    		fileToProcess.close();
    	} catch (IOException e) {
    		e.printStackTrace(System.err);
    	}
    }

	@SuppressWarnings("unchecked")
    private void constructHeader() {
		ArrayList<String> header = new ArrayList<String>();
		
        List<String> variableNames = studyXMLHandler.getVariableNames();
        String variableNamesHeader = composeValuesAsOneString(variableNames);
        header.add(variableNamesHeader);

		basecase = studyXMLHandler.getBaseCaseFileName();
		PythagorasMOEHeader pythMOES = new PythagorasMOEHeader(basecase.toString());
        String moeHeader = pythMOES.getHeader(delimiter);
		header.add(moeHeader);
        
        header.add(0, "Identifier");
        if (studyXMLHandler.isExcursionElementExist("Fitness")) {
        	header.add("Fitness");
            dofitness = true;
        }

        headerLine = composeValuesAsOneString(header);
        
        try {
        	logFileWriter.write("Variable header: " + variableNamesHeader);
        	logFileWriter.write(lineSeparator);
        	logFileWriter.write("MOE header: " + moeHeader);
        	logFileWriter.write(lineSeparator);

        } catch (IOException e) {
        	e.printStackTrace(System.err);
        }
 
    }
    

    private void printData() {
        try {
            FileWriter fw = new FileWriter(dataFile.toString(), true);
            //write the header
            fw.write(headerLine);
            fw.write(lineSeparator);
            for (int i = 0; i < dataList.size(); i++) {
                String dataLine = (String) dataList.get(i);
                fw.write(dataLine);
                fw.write(lineSeparator);
            }
            fw.close();
        } catch (IOException ioe) {
            System.out.println("Error attempting to open: " + dataFile.toString());
            ioe.printStackTrace(System.out);
            System.exit(-1);
        }
    }
    
    public static String printUsageMessage() {
    	final String[] message = {
    		"Usage: java oldmcdata.model.pythagoras.postprocess.PythagorasPostProcessor ",
    		"studyDir studyFileName dataFileName prependIdentifier",
    		" ", 
    		"where studyDir is the directory that contains the study file, and the MOE directory",
    		"dataFileName is a filename where you wish to have all the data printed to",
    		"prependIdentifier is an optional text string to prepend to each data line",
    		"-multiple can be specified as the final argument, to use if the MOE results ",
    		"for each excursion are in a single file."
    	};
    	String lineSep = System.getProperty("line.separator");
    	StringBuilder usageMessage = new StringBuilder();
    	for (String line : message ) {
    		usageMessage.append(line).append(lineSep);
    	}
    	
    	return usageMessage.toString();
    }

    public static void main(String[] args) {
        PythagorasPostProcessor postprocessor = null;
        System.out.println("Running PythagorasPostProcessor");
        switch (args.length) {
            case 3:
                postprocessor = new PythagorasPostProcessor(args[0], args[1], args[2]);
                postprocessor.postProcess();
                break;
            case 4:
                postprocessor = new PythagorasPostProcessor(args[0], args[1], args[2]);
                if (args[3].equalsIgnoreCase("-multiple"))
                	postprocessor.setMultipleRepsPerFile(true);
                else
                	postprocessor.setPrependIdentifier(args[3]);
                postprocessor.postProcess();
                break;
            case 5:
                postprocessor = new PythagorasPostProcessor(args[0], args[1], args[2]);
                if (args[4].equalsIgnoreCase("-multiple")) {
                	postprocessor.setMultipleRepsPerFile(true);
                	postprocessor.setPrependIdentifier(args[3]);
                }
                else {
                	System.out.println(PythagorasPostProcessor.printUsageMessage());
                	System.out.println("It appears you may have the -multiple option " + 
                			"specified prior to the prepend option.");
                	System.exit(-1);
                }
                postprocessor.postProcess();
            	
            	
            	break;
            default:
                System.out.println(PythagorasPostProcessor.printUsageMessage());
                break;
        }
        System.out.println("Completed PythagorasPostProcessor");
    }
}