package oldmcdata.postprocessors.model;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import oldmcdata.analyzers.utils.OutputDataFileHandler;
import oldmcdata.utils.DefaultStudyXMLHandler;
import oldmcdata.utils.StudyXML;
import oldmcdata.utils.StudyXMLHandler;

import org.dom4j.Element;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class SocratesPostProcessor {
    protected OutputDataFileHandler outputDataFileHandler;
    protected BufferedReader fileToProcess;
    protected StudyXMLHandler studyXMLHandler;
    private String headerLine, variableNamesHeader, delimiter = ",", lineSeparator,
    prependIdentifier;
    private int numberOfLinesToHeader = 0;
    private int numberOfReplicates;
    private List dataList, iterationElements, excursions, moeFileNames;
    private File dataFile;
    private boolean doPrepend = false, dofitness = false;
    
	private File basecase;
	private Element moeOutput;

    public SocratesPostProcessor(String studyDirName, String studyFileName, String dataFileName) {
        studyXMLHandler = new DefaultStudyXMLHandler(studyDirName, studyFileName);
        outputDataFileHandler = new OutputDataFileHandler();
        dataFile = new File(studyXMLHandler.getStudyDirectory(), dataFileName);
        lineSeparator = System.getProperty("line.separator");
    }

    public void setPrependIdentifier(String prepend) {
        prependIdentifier = prepend;
        doPrepend = true;
    }

    public void postProcess() {
        studyXMLHandler.openStudyXMLDocument();
        iterationElements = studyXMLHandler.getIterations();
        numberOfReplicates = Integer.parseInt(studyXMLHandler.getNumberReplicates());
        processFirstIteration();
        for (int i = 1; i < iterationElements.size(); i++) {
            Element iterationElement = (Element) iterationElements.get(i);
            processIteration(iterationElement);
            // print iteration at a time, assuming all of the data would not fit into memory
        }
    }

    private void processFirstIteration() {
        Element iterationElement = (Element) iterationElements.get(0);
        excursions = studyXMLHandler.getExcursions(iterationElement);
        moeFileNames = studyXMLHandler.getMOEFileNames(iterationElement);
        dataList = new ArrayList();
        String moeFileName = (String) moeFileNames.get(0);
        printHeader(moeFileName);
        processExcursions();
        // print iteration at a time, assuming all of the data would not fit into memory
        printData();
    }

    private void processIteration(Element iterationElement) {
        dataList = new ArrayList();
        excursions = studyXMLHandler.getExcursions(iterationElement);
        moeFileNames = studyXMLHandler.getMOEFileNames(iterationElement);
        processExcursions();
        printData();
    }

    private void processExcursions() {
        for (int j = 0; j < excursions.size(); j++) {
            Element excursion = (Element) excursions.get(j);
            String moeFileName = (String) moeFileNames.get(j);
            processExcursion(excursion, moeFileName);
        }
    }

    private void processExcursion(Element excursion, String moeFileName) {
        List excursionValues = new ArrayList();
        String excursionID = excursion.attributeValue(StudyXML.ID);
        String iterationID = excursion.getParent().attributeValue(StudyXML.ID);
        StringBuffer sb = new StringBuffer("I").append(iterationID);
        sb.append("E").append(excursionID);
        if (doPrepend)
            sb.insert(0, prependIdentifier);
        excursionValues.add(sb.toString());
        excursionValues.addAll(studyXMLHandler.getExcursionValues(excursion));
        if (dofitness) {
            String fitness = excursion.element(StudyXML.FITNESS).getTextTrim();
            excursionValues.add(fitness);
        }
        String values = composeValuesAsOneString(excursionValues);
        fileToProcess = outputDataFileHandler.getDataReader(moeFileName);
        processData(values);
    }

    private String composeValuesAsOneString(List values) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < values.size(); i++) {
            String vName = (String) values.get(i);
            sb.append(vName).append(delimiter);
        }
		sb.deleteCharAt(sb.length()-1);
		return sb.toString();
    }

    private void skipNumberOfLines(int numberOfLinesToSkip) {
        try {
            for (int i = 0; i < numberOfLinesToSkip; i++) {
                String dataLine = fileToProcess.readLine();
            }
        } catch (IOException e) {
            e.printStackTrace(System.out);
        }
    }

    protected void processData(String prependString) {
        try {
            for (int i = 0; i < numberOfReplicates; i++) {
                String dataLine = fileToProcess.readLine();
                dataList.add(prependString + dataLine);
            }
            fileToProcess.close();
        } catch (IOException e) {
            e.printStackTrace(System.out);
        }
    }

    private void printHeader(String moeFileName) {
        List variableNames = studyXMLHandler.getVariableNames();
        variableNames.add(0, "Identifier");
        if (studyXMLHandler.isExcursionElementExist("Fitness")) {
            variableNames.add("Fitness");
            dofitness = true;
        }
        composeMOENames(variableNames);
        variableNamesHeader = composeValuesAsOneString(variableNames);
        dataList.add(variableNamesHeader);

    }
    
	private void composeMOENames(List variableNames) {
		variableNames.add("End Time");
		variableNames.add("Index");
		variableNames.add("seed");
		variableNames.add("DataFarmingString");
		variableNames.add("Red Dead");
		variableNames.add("Blue Dead");
		variableNames.add("Time 100% Red Dead");
		variableNames.add("Time 75% Red Dead");
		variableNames.add("Time 50% Red Dead");
		variableNames.add("Time 25% Red Dead");
		variableNames.add("Time 100% Blue Dead");
		variableNames.add("Time 75% Blue Dead");
		variableNames.add("Time 50% Blue Dead");
		variableNames.add("Time 25% Blue Dead");	
		variableNames.add("Time First Blue Dead");
		variableNames.add("Time First Red Dead");
		variableNames.add("Number Red T0 decisions");
		variableNames.add("Number Blue T0 decisions");	
		variableNames.add("Time First Greed Dead");		
		variableNames.add("Number Yellow Agents");	
		variableNames.add("Yellow Dead");	
		variableNames.add("Time First Yellow Dead");			
   	
	}
    

    private void printData() {
        try {
            FileWriter fw = new FileWriter(dataFile.toString(), true);
            for (int i = 0; i < dataList.size(); i++) {
                String dataLine = (String) dataList.get(i);
                fw.write(dataLine);
                fw.write(lineSeparator);
            }
            fw.close();
        } catch (IOException ioe) {
            System.out.println("Error attempting to open: " + dataFile.toString());
            ioe.printStackTrace(System.out);
            System.exit(-1);
        }
    }

    public static void main(String[] args) {
        SocratesPostProcessor postprocessor = null;
        System.out.println("Running SocratesPostProcessor");
        switch (args.length) {
            case 3:
                postprocessor = new SocratesPostProcessor(args[0], args[1], args[2]);
                postprocessor.postProcess();
                break;
            case 4:
                postprocessor = new SocratesPostProcessor(args[0], args[1], args[2]);
                postprocessor.setPrependIdentifier(args[3]);
                postprocessor.postProcess();
                break;
            default:
                System.out.println("Usage: java oldmcdata.model.socrates.postprocess.SocratesPostProcessor studyDir studyFileName dataFileName prependIdentifier");
                System.out.println("studyDir is the directory that contains the study file, and the MOE directory");
                System.out.println("dataFileName is a filename where you wish to have all the data printed to");
                System.out.println("prependIdentifier is an optional text string to prepend to each data line");
                break;
        }
    }
}