package oldmcdata.utils;


import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class DefaultStudyXMLHandler implements StudyXMLHandler {
    private File studyDir;
    private File studyFile;
    private String studyFileName;
    private Document studyDoc;
    // listing of all the major elements
    private Element rootElement,
    modelParameters,
    submissionParameters,
    modelIdentification,
    modelVersion,
    algorithm,
    modelRunInformation,
    algorithmSpecification,
    generatorAlgorithm,
    analyzerAlgorithm,
    evaluator;


    public DefaultStudyXMLHandler(String studyDirName, String studyFileName) {
        studyDir = new File(studyDirName);
        studyFile = new File(studyDir, studyFileName);
        this.studyFileName = studyFileName;
    }

    public void openStudyXMLDocument() {
        try {
            SAXReader xmlReader = new SAXReader();
            xmlReader.setStripWhitespaceText(true);
            studyDoc = xmlReader.read(studyFile);
            rootElement = studyDoc.getRootElement();
            setAlgorithmElement();
            setModelIdentificationElement();
            setModelParametersElement();
            setModelRunInformationElement();
            setSubmissionParametersElement();
            setAlgorithmSpecificationElement();
            setEvaluatorElement();
            setGeneratorAlgorithmElement();
            setAnalyzerAlgorithmElement();
        } 
//            catch (MalformedURLException mue) {
//            System.out.println("Can not form URL from this file: " + studyFile);
//            mue.printStackTrace(System.out);
//            System.exit(-1);
//        } 
            catch (DocumentException de) {
            System.out.println("Error attempting to parse: " + studyFile);
            de.printStackTrace(System.out);
            System.exit(-1);
        }

    }

    // setters
    public void setAlgorithmElement() {
        algorithm = rootElement.element(StudyXML.ALGORITHM);
    }

    public void setSubmissionParametersElement() {
        submissionParameters = rootElement.element(StudyXML.SUBMISSIONPARAMETERS);
    }

    public void setModelParametersElement() {
        modelParameters = rootElement.element(StudyXML.MODELPARAMETERS);
    }

    public void setModelIdentificationElement() {
        modelIdentification = rootElement.element(StudyXML.MODELIDENTIFICATION);
        modelVersion = modelIdentification.element(StudyXML.MODELVERSION);
    }

    public void setEvaluatorElement() {
        evaluator = submissionParameters.element(StudyXML.EVALUATOR);
    }

    public void setAlgorithmSpecificationElement() {
        algorithmSpecification = algorithm.element(StudyXML.ALGORITHMSPECIFICATION);
    }

    public void setGeneratorAlgorithmElement() {
        generatorAlgorithm = algorithmSpecification.element(StudyXML.GENERATORALGORITHM);
    }

    public void setAnalyzerAlgorithmElement() {
        analyzerAlgorithm = algorithmSpecification.element(StudyXML.ANALYZERALGORITHM);
    }

    public void setModelRunInformationElement() {
        modelRunInformation = algorithm.element(StudyXML.MODELRUNINFORMATION);
    }


    // getters
    public File getStudyDirectory() {
        return studyDir;
    }

    public File getExcursionDirectory() {
        Element iteration = getMostRecentIterationElement();
        Element iterationExcursionFileInfo = iteration.element(StudyXML.EXCURSIONFILEINFO);
        String excursionDirName = iterationExcursionFileInfo.element(StudyXML.EXCURSIONDIR).getTextTrim();
        File excursionDir = new File(studyDir, excursionDirName);
        return excursionDir;
    }

    public File getPlaybackDirectory() {
        Element iteration = getMostRecentIterationElement();
        Element iterationExcursionFileInfo = iteration.element(StudyXML.EXCURSIONFILEINFO);
        String playbackDirName = iterationExcursionFileInfo.element(StudyXML.PLAYBACKDIR).getTextTrim();
        File playbackDir = new File(studyDir, playbackDirName);
        return playbackDir;
    }
    
    public File getMOEDirectory() {
        Element iteration = getMostRecentIterationElement();
        Element iterationExcursionFileInfo = iteration.element(StudyXML.EXCURSIONFILEINFO);
        String moeDirName = iterationExcursionFileInfo.element(StudyXML.MOEDIR).getTextTrim();
        File moeDir = new File(studyDir, moeDirName);
        return moeDir;
    }
    
    public File getStudyFile() {
        return studyFile;
    }

    public String getStudyFileName() {
        return studyFileName;
    }

    public Element getAlgorithmElement() {
        return algorithm;
    }

    public Element getGeneratorAlgorithmElement() {
        return generatorAlgorithm;
    }

    public Element getAnalyzerAlgorithmElement() {
        return analyzerAlgorithm;
    }

    public Element getAlgorithmSpecificationElement() {
        return algorithmSpecification;
    }

    public Element getModelRunInformationElement() {
        return modelRunInformation;
    }

    public Element getModelParametersElement() {
        return modelParameters;
    }

    public Element getSubmissionParametersElement() {
        return submissionParameters;
    }

    public Element getModelIdentificationElement() {
        return modelIdentification;
    }

    public Element getGeneratorAlgorithmParameters() {
        return generatorAlgorithm.element(StudyXML.PARAMETERS);
    }

    public String getGeneratorAlgorithmClassName() {
        if (generatorAlgorithm == null)
            return null;
        else
            return generatorAlgorithm.attributeValue(StudyXML.CLASSNAME);
    }
    
    public File getBaseCaseFileName() {
		Element basecase = generatorAlgorithm.element(StudyXML.EXCURSIONFILEINFO).element(StudyXML.BASECASEFILENAME);
		return new File(getStudyDirectory(), basecase.getText());
    }

    public String getAnalyzerAlgorithmClassName() {
        if (analyzerAlgorithm == null)
            return null;
        else
            return analyzerAlgorithm.attributeValue(StudyXML.CLASSNAME);
    }

    public List getVariables() {
        return generatorAlgorithm.element(StudyXML.DIMENSIONS).elements(StudyXML.VARIABLE);
    }

    public List getVariableNames() {
        List variableElements = getVariables();
        List variableNames = new ArrayList();
        for (int i = 0; i < variableElements.size(); i++) {
            Element variable = (Element) variableElements.get(i);
            variableNames.add(variable.attributeValue(StudyXML.NAME));
        }
        return variableNames;
    }

    public List getIterations() {
        return modelRunInformation.elements(StudyXML.ITERATION);
    }

    public List getExcursions(Element iterationElement) {
        return iterationElement.elements(StudyXML.EXCURSION);
    }

    /** returns the values of the Excursion element as a List of Strings
     */
    public List getExcursionValues(Element excursion) {
        List excursionValues = new ArrayList();
        List valueElements = excursion.elements(StudyXML.VALUE);
        for (int i = 0; i < valueElements.size(); i++) {
            Element valueElement = (Element) valueElements.get(i);
            excursionValues.add(valueElement.getText());
        }
        return excursionValues;
    }

    public Element getIteration(String iterationID) {
        Element iterationRequested = null;
		StringBuffer xpathString = new StringBuffer("/Iteration[@id='");
		xpathString.append(iterationID).append("']");
		XPath xpath = DocumentHelper.createXPath(xpathString.toString());
        Node iterationNode = xpath.selectSingleNode(modelRunInformation);
        if (iterationNode == null)
            System.out.println("Can not locate iteration " + iterationID);
        else
            iterationRequested = (Element) iterationNode;

        return iterationRequested;
    }
    

    public Element getExcursion(String iterationID, String excursionID) {
        Element excurisonRequested = null;
		StringBuffer xpathString = new StringBuffer("/Iteration[@id='");
		xpathString.append(iterationID).append("']");
		xpathString.append("/Excursion[@id='");
		xpathString.append(excursionID).append("']");
		XPath xpath = DocumentHelper.createXPath(xpathString.toString());
        Node excursionNode = xpath.selectSingleNode(modelRunInformation);
        if (excursionNode == null)
            System.out.println("Can not locate iteration/excursion " + iterationID + ":" + excursionID);
        else
            excurisonRequested = (Element) excursionNode;

        return excurisonRequested;
    }

    public Element getMostRecentIterationElement() {
        List iterations = modelRunInformation.elements(StudyXML.ITERATION);
        int size = iterations.size();
        if (size == 0)
            return null;
        else
            return (Element) iterations.get(size - 1);
    }

    public List getMostRecentExcursions() {
        Element currentIteration = getMostRecentIterationElement();
        if (currentIteration == null)
            return null;
        else
            return currentIteration.elements(StudyXML.EXCURSION);
    }

    public List getMOEFileNames() {
        return getMOEFileNames(getMostRecentIterationElement());
    }

    public List getMOEFileNames(Element iterationElement) {
        List<String> moeFileNames = new ArrayList<String>();
        int numberOfExcursions = iterationElement.elements(StudyXML.EXCURSION).size();
        Element excursionFileInfo = iterationElement.element(StudyXML.EXCURSIONFILEINFO);
        String moeDirName = excursionFileInfo.element(StudyXML.MOEDIR).getTextTrim();
        File moeDir = new File(studyDir, moeDirName);
        String moeFileStub = excursionFileInfo.element(StudyXML.MOEFILESTUB).getTextTrim();
        for (int i = 0; i < numberOfExcursions; i++) {
            String moeFileName = moeFileStub + Integer.toString(i);
            File moeFile = new File(moeDir, moeFileName);
            moeFileNames.add(moeFile.toString());
        }
        return moeFileNames;
    }

    public boolean isExcursionElementExist(String elementName) {
        String xpath = "//Iteration/Excursion/" + elementName;
        Node excursionElementNode = rootElement.selectSingleNode(xpath);
        if (excursionElementNode == null) {
//            System.out.println("can't find excursionElement: " + elementName);
//            System.out.println("xpath expression: " + xpath);
            return false;
        } else {
            System.out.println("did find excursionElement: " + elementName);
            System.out.println("xpath expression: " + xpath);
            return true;
        }
    }

    public void clearMostRecentIterationElement() {
        Element iteration = getMostRecentIterationElement();
        Element parent = iteration.getParent();
        parent.remove(iteration);
    }

    public String getEvaluatorClassName() {
        if (evaluator == null)
            return null;
        else
            return evaluator.attributeValue(StudyXML.CLASSNAME);
    }

    public Element getEvaluatorParameters() {
        return evaluator.element(StudyXML.PARAMETERS);
    }

    public String getNumberReplicates() {
        return modelParameters.element(StudyXML.NUMBERREPLICATES).getTextTrim();
    }

    public List getInitialSeeds() {
        List seedList = modelParameters.elements(StudyXML.INITIALRANDOMSEED);
        List seeds = new ArrayList();
        for (int i = 0; i < seedList.size(); i++) {
            Element seed = (Element) seedList.get(i);
            seeds.add(new String(seed.getTextTrim()));
        }
        return seeds;
    }

    public String getModelName() {
        return modelIdentification.element(StudyXML.MODELNAME).getTextTrim();
    }

    public String getModelMajorVersion() {
        return modelVersion.element("Major").getTextTrim();
    }

    public String getModelMinorVersion() {
        return modelVersion.element("Minor").getTextTrim();
    }

    public String getModelVersion() {
        String majorVersion = modelVersion.element("Major").getTextTrim();
        String minorVersion = modelVersion.element("Minor").getTextTrim();
        return majorVersion + "." + minorVersion;
    }

    public synchronized void writeStudyXMLDocument() {
        try {
            FileOutputStream fw = new FileOutputStream(studyFile);
            XMLWriter xmlWriter = new XMLWriter(fw, OutputFormat.createPrettyPrint());
            xmlWriter.write(studyDoc);
            xmlWriter.close();
        } catch (IOException ioe) {
            System.out.println("Error attempting to open: " + studyFile);
            ioe.printStackTrace(System.out);
            System.exit(-1);
        }
    }


}
