package oldmcdata.utils;


import java.text.NumberFormat;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class DefaultTranslator implements Translator {
    private final static String DEFAULTDELIMITER = ",";
    private Element iteration;
    private Element generatorAlgorithm;
    private StudyXMLHandler studyXMLHandler;
    private List variables;
    private List excursionsList;
    private int iterationCount = 0;
    private int excursionCount;
    private String delimiter = DEFAULTDELIMITER;

    public DefaultTranslator() {
    }

    public void initialize() {
        System.out.println(this.getClass().getName() + ":initialize()");
    }

    public void setDelimiter(String d) {
        delimiter = d;
    }

    public String getDelimiter() {
        return delimiter;
    }

    public void translate(int iterationCount, List excursionsList, StudyXMLHandler studyXMLHandler) {
        this.iterationCount = iterationCount;
        translate(excursionsList, studyXMLHandler);
    }

    public void translate(List excursionsList, StudyXMLHandler studyXMLHandler) {
        System.out.println(this.getClass().getName() + ":translate()");
        this.studyXMLHandler = studyXMLHandler;
        this.excursionsList = excursionsList;
        generatorAlgorithm = studyXMLHandler.getGeneratorAlgorithmElement();

        buildIterationElement();
        buildVariableElements();
        buildExcursionFileInfo();
        buildExcursions();
    }

    private void buildIterationElement() {
        Element modelRunInformation = studyXMLHandler.getModelRunInformationElement();
        iteration = DocumentHelper.createElement(StudyXML.ITERATION);
        iteration.addAttribute("id", Integer.toString(iterationCount));
        modelRunInformation.add(iteration);
    }

    private void buildVariableElements() {
        variables = studyXMLHandler.getVariables();
        ;
        Iterator varIt = variables.iterator();
        while (varIt.hasNext()) {
            Element var = (Element) varIt.next();
            Element variable = var.createCopy();
            iteration.add(variable);
        }

    }

    /** Builds the ExcursionFileInfo element for the current Iteration, based on the template
     provided in the corresponding element in the GeneratorAlgorithm element. First creates a copy using
     the template, then sets the ExcursionFileStub, MOEFileStub, and PlaybackFileStub elements using
     the method indicated in the buildStub method.
     */
    private void buildExcursionFileInfo() {
        Element excursionInfoTemplate = generatorAlgorithm.element(StudyXML.EXCURSIONFILEINFO);
        Element basecaseFileElement = excursionInfoTemplate.element(StudyXML.BASECASEFILENAME);
        // take the first elements of the file name up to the first "."
//        StringTokenizer st = new StringTokenizer(basecaseFileElement.getTextTrim(), ".");
//        String basecaseFileName = st.nextToken(); //take the first token as the root for the basecase
        Element excursionFileStubElement = excursionInfoTemplate.element(StudyXML.EXCURSIONFILESTUB);
//        excursionFileStubElement.setText(basecaseFileName + ".");
        Element MOEFileStubElement = excursionInfoTemplate.element(StudyXML.MOEFILESTUB);
//        MOEFileStubElement.setText(basecaseFileName + "-MOE.");

        // now build up excursionFileInfo for current Iteration
        Element excursionInfo = excursionInfoTemplate.createCopy();
        basecaseFileElement = excursionInfo.element(StudyXML.BASECASEFILENAME);
//        basecaseFileElement.setText(basecaseFileName + ".");
        buildStub(basecaseFileElement, false);
        excursionFileStubElement = excursionInfo.element(StudyXML.EXCURSIONFILESTUB);
        buildStub(excursionFileStubElement, true);
        MOEFileStubElement = excursionInfo.element(StudyXML.MOEFILESTUB);
        buildStub(MOEFileStubElement, true);
        Element PlaybackFileStubElement = excursionInfo.element(StudyXML.PLAYBACKFILESTUB);
        buildStub(PlaybackFileStubElement, true);
        iteration.add(excursionInfo);
    }

    /** Builds the file stub element by progressively concatenating the stub template, which is passed in to the method,
     the GeneratorAlgorithm name, a hyphen, and the iteration count. Finally appends a ".".
     */
    private void buildStub(Element stub, boolean append) {
        String generatorAlgorithmName = generatorAlgorithm.attributeValue(StudyXML.ALGORITHMNAME);
        StringBuffer stubStub = new StringBuffer(stub.getText());
//        stubStub.append(generatorAlgorithmName);
        stubStub.append("-");
        stubStub.append(Integer.toString(iterationCount));
        if (append)
            stubStub.append("-");
        stub.setText(stubStub.toString());
    }

    /** Builds excursions from a List. Assumes excursions are in the form of a string,
     delimited by the String specified by the delimiter parameter.
     */
    private void buildExcursions() {
        Iterator excursionsIterator = excursionsList.iterator();

        excursionCount = 0;
        while (excursionsIterator.hasNext()) {
            String s = (String) excursionsIterator.next();
            buildExcursion(s);
        }

    }

    /** Builds an excursion from a String. Assumes an excursion is in the form of a string,
     with each value delimited by the String specified by the delimiter parameter.
     For every value, creates
     a Value element, and adds that to the Excursion element. The Excursion element is then added
     to the Iteration element,with an accompanying id attribute set to the excursion count.
     */
    private void buildExcursion(String s) {
        String[] values;

        NumberFormat nf = NumberFormat.getInstance();
        Element excursion = DocumentHelper.createElement(StudyXML.EXCURSION);
        StringTokenizer st = new StringTokenizer(s, delimiter);
        int numTokens = st.countTokens();
        values = new String[numTokens];
        for (int i = 0; i < numTokens; i++)
            values[i] = st.nextToken();


        for (int i = 0; i < variables.size(); i++) {
            Element variable = (Element) variables.get(i);
            Element value = DocumentHelper.createElement(StudyXML.VALUE);
            String type = variable.attributeValue(StudyXML.TYPE);
            if (type.equalsIgnoreCase("float")) {// truncate to 2 decimal places
                Double val = new Double(values[i]);
                nf.setMaximumFractionDigits(2);
                value.addText(nf.format(val.doubleValue()));
            }
            else if (type.equalsIgnoreCase("integer") || type.equalsIgnoreCase("int") ) {
            	Double val = new Double(values[i]);
                nf.setMaximumFractionDigits(0);
                value.addText(nf.format(val));
            }
            else   // string take as is
                value.addText(values[i]);

            value.addAttribute(StudyXML.TYPE, type);
            excursion.add(value);
        }

        excursion.addAttribute("id", Integer.toString(excursionCount++));
        iteration.add(excursion);
    }
}