package oldmcdata.utils;


import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;


/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class ExcursionFileCreator {
    private final String excursionFileSuffix = ".xml",
    moeFileSuffix = "",
    playbackFileSuffix = ".moves";
    private final String EXCURSION = "ExcursionFile",
    MOE = "MoeFile",
    PLAYBACK = "PlaybackFile";
    private StudyXMLHandler studyXMLHandler;
    private Document basecaseDoc;
    private Element iteration, excursionFileInfo;
    private Map directories, argumentMap;
    private List variables, excursionElements;
    private List excursionFiles, moeFiles, playbackFiles;
    private int numberOfExcursions;
	private FileWriter logFileWriter;
	private final static String lineSeparator = System.getProperty("line.separator");
	private List<String> logEntries;


    public ExcursionFileCreator() {
        directories = new HashMap();
        argumentMap = new HashMap();
        excursionFiles = new ArrayList();
        moeFiles = new ArrayList();
        playbackFiles = new ArrayList();
        logEntries = new ArrayList<String>();
    }

    public List getExcursionFiles() {
        return excursionFiles;
    }

    public List getMOEFiles() {
        return moeFiles;
    }

    public List getPlaybackFiles() {
        return playbackFiles;
    }

    public Map getArgumentMap() {
        return argumentMap;
    }

    public int getNumberOfExcursions() {
        return numberOfExcursions;
    }

    /** Creates the Excursion files using the basecase as the template for all excursions in the
     current iteration.
     */
    public void createExcursions(StudyXMLHandler studyXMLHandler) {
        clearLists();
        this.studyXMLHandler = studyXMLHandler;
        studyXMLHandler.openStudyXMLDocument();
        // get the data to use to create the directories and the excursions
        iteration = studyXMLHandler.getMostRecentIterationElement();
        excursionFileInfo = iteration.element(StudyXML.EXCURSIONFILEINFO);
        variables = iteration.elements(StudyXML.VARIABLE);
        excursionElements = iteration.elements(StudyXML.EXCURSION);

        openBaseCaseDocument();
        validateVariables();
        createDirectories();
        createExcursions();
        buildArgumentMap();

    }

    private void validateVariables() {
        List xpaths, nullXPaths;
        nullXPaths = new ArrayList();
        Element root = basecaseDoc.getRootElement();
        for (int i = 0; i < variables.size(); i++) {
            Element variable = (Element) variables.get(i);
            xpaths = variable.elements(StudyXML.XPATH);
            for (int j = 0; j < xpaths.size(); j++) {
                Element xpath = (Element) xpaths.get(j);
                Node nodeToChange = root.selectSingleNode(xpath.getText());
                if (nodeToChange == null)
                    nullXPaths.add(xpath.getText());
            }
        }
        if (nullXPaths.size() != 0) {
            System.err.println();
            System.err.println("***************    ERROR PROCESSING VARIABLES ***************");
            System.err.println("Can't find the below listed variables in scenario file");
            for (Iterator it = nullXPaths.iterator(); it.hasNext();) {
                String xpathtext = (String) it.next();
                System.err.println(xpathtext);
            }
            System.err.println("Quitting *************");
            studyXMLHandler.clearMostRecentIterationElement();
            studyXMLHandler.writeStudyXMLDocument();
            System.exit(-1);
        }
    }


    /** Clears entries to the lists that were added from previous iterations.
     */
    private void clearLists() {
        excursionFiles.clear();
        moeFiles.clear();
        playbackFiles.clear();
        argumentMap.clear();
    }

    /** Creates the Excursion, MOE, and Playback directories, if they do not exist.
     */
    private void createDirectories() {
        createDirectory(excursionFileInfo.element(StudyXML.EXCURSIONDIR));
        createDirectory(excursionFileInfo.element(StudyXML.MOEDIR));
        createDirectory(excursionFileInfo.element(StudyXML.PLAYBACKDIR));
    }

    private void createDirectory(Element dirToCreate) {
        File directory = new File(studyXMLHandler.getStudyDirectory(), dirToCreate.getTextTrim());
        directory.mkdir();
        directories.put(dirToCreate.getName(), directory);
    }

    public void openBaseCaseDocument() {
        String basecaseFileName = excursionFileInfo.element(StudyXML.BASECASEFILENAME).getText();
        File basecaseFile = new File(studyXMLHandler.getStudyDirectory(), basecaseFileName);
        try {
            SAXReader xmlReader = new SAXReader();
            xmlReader.setStripWhitespaceText(false);
            basecaseDoc = xmlReader.read(basecaseFile);

        } 
//        catch (MalformedURLException mue) {
//            System.out.println("Can not form URL from this file: " + basecaseFile);
//            mue.printStackTrace(System.out);
//            System.exit(-1);
//        } 
        catch (DocumentException de) {
            System.out.println("Error attempting to parse: " + basecaseFile);
            de.printStackTrace(System.out);
            System.exit(-1);
        }

    }


    /** Iterates through the excursion list to create the individual excursion xml files. Also creates
     and adds the Files to Excursion, MOE and Playback lists.
     */
    private void createExcursions() {
        String excursionFileStub = excursionFileInfo.element(StudyXML.EXCURSIONFILESTUB).getText();
        String moeFileStub = excursionFileInfo.element(StudyXML.MOEFILESTUB).getText();
        String playbackFileStub = excursionFileInfo.element(StudyXML.PLAYBACKFILESTUB).getText();

        Iterator excursionIterator = excursionElements.iterator();
        while (excursionIterator.hasNext()) {
            Element excursion = (Element) excursionIterator.next();
            File excursionFile = createFile(excursion, StudyXML.EXCURSIONDIR, excursionFileStub, excursionFileSuffix);
            File moeFile = createFile(excursion, StudyXML.MOEDIR, moeFileStub, moeFileSuffix);
            File playbackFile = createFile(excursion, StudyXML.PLAYBACKDIR, playbackFileStub, playbackFileSuffix);

            excursionFiles.add(excursionFile);
            moeFiles.add(moeFile);
            playbackFiles.add(playbackFile);

            createExcursion(excursion, excursionFile);
        }
        numberOfExcursions = excursionFiles.size();
    }

    /** Creates a filename using the filestub, the excursions ID, and the corresponding filesuffix. Then
     prepends the associated directory.
     */
    private File createFile(Element excursion, String directory, String fileStub, String fileSuffix) {
        StringBuffer fileName = new StringBuffer(fileStub).
                append(excursion.attributeValue(StudyXML.ID)).
                append(fileSuffix);
        return new File((File) directories.get(directory), fileName.toString());
    }

    /** Creates the individual Excursion xml file. Clones the basecase, then makes changes to the nodes
     specified by the variable list. Finally, writes out the xml document to the previously created excursion
     File.
     */
    private void createExcursion(Element excursion, File excursionFile) {
        Document excursionDoc = (Document) basecaseDoc.clone();
        Element root = excursionDoc.getRootElement();
        List values = excursion.elements(StudyXML.VALUE);

        logEntries.clear();
        logEntries.add("Excursion: " + excursionFile.getName() + "*********************");
		for (int i = 0; i < variables.size(); i++) {
			Element variable = (Element) variables.get(i);
			Element value = (Element) values.get(i);
			String valueText = value.getTextTrim();
			List xpaths = variable.elements(StudyXML.XPATH);
			for (int j = 0; j < xpaths.size(); j++) {
				Element xpath = (Element) xpaths.get(j);
				List nodesToChange = root.selectNodes(xpath.getText());
				for (int k=0;k<nodesToChange.size();k++) {
					Node nodeToChange = (Node)nodesToChange.get(k);
					nodeToChange.setText(valueText);
					logEntries.add(nodeToChange.getUniquePath() + ":" + valueText);
				}
				
				
			}
		}

        writeExcursionDocument(excursionFile, excursionDoc);
        writeLog();
    }

    /** Writes out the excursion document to the file indicated by excursionFile. The output format
     (OutputFormat.createPrettyPrint() )
     uses an indent of 2, adds newlines between element tags, and strips off leading and trailing whitespace
     within elements.
     */
    public void writeExcursionDocument(File excursionFile, Document excursionDoc) {
        try {
            FileWriter fw = new FileWriter(excursionFile);
//            XMLWriter xmlWriter = new XMLWriter(fw, OutputFormat.createPrettyPrint());
            OutputFormat of = OutputFormat.createCompactFormat();
            of.setTrimText(false);
            XMLWriter xmlWriter = new XMLWriter(fw, of);
            xmlWriter.write(excursionDoc);
            xmlWriter.close();
        } catch (IOException ioe) {
            System.out.println("Error attempting to open: " + excursionFile);
            ioe.printStackTrace(System.out);
            System.exit(-1);
        }
    }

    private void buildArgumentMap() {
        argumentMap.put(EXCURSION, excursionFiles.iterator());
        argumentMap.put(MOE, moeFiles.iterator());
        argumentMap.put(PLAYBACK, playbackFiles.iterator());
    }
    
	public void writeLog() {
		openLogFile();
		try {
			for (String entry : logEntries ){
				logFileWriter.write(entry);
				logFileWriter.write(lineSeparator);
			}
		}
		catch(IOException io ) {
			System.err.println("Attempted to write events to output file in Logger");
			io.printStackTrace(System.err);
		}

		closeLogFile();
	}
    
	public void openLogFile() {
		File outFile = null;
//		String[] splits = scenarioFileName.split("\\.");
//		String rootFileName = splits[0];
		String outFileName = "ExcursionLogFileCreator.log";
		outFile = new File(studyXMLHandler.getStudyDirectory(), outFileName);
		try {
			// create a new file if this one doesn't already exist
			outFile.createNewFile();
			// append to the current file
			logFileWriter = new FileWriter(outFile, true);
		}
		catch(IOException io) {
			System.err.println("Attempted to create output file in Logger");
			io.printStackTrace(System.err);
		}
		
	}
	
	public void closeLogFile() {
		try {
			logFileWriter.close();
		}
		catch(IOException io) {
			System.err.println("Attempted to close output file in Logger");
			io.printStackTrace(System.err);
		}

	}


}
