/**
 * 
 */
package oldmcdata.utils.xml;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;

/**
 * @author stephenupton
 *
 */
public class FileHandler {
	private File inputDirectory;
    private File directoryToWriteFiles;
    private Element filesElement;
    private ArrayList<File> filesToChange;
    private Pattern filePattern;
	
	public FileHandler(Element e) {
		inputDirectory = new File(System.getProperty("user.dir")); //default
		filesElement = e;
		filesToChange = new ArrayList<File>();
	}
	
	public ArrayList<File> getFilesToChange() {
		filesToChange.clear();
		setInputDirectory();
		processFileSet();
		
		return filesToChange;
	}
	
	private void processFileSet() {
		Element fileSet = filesElement.element("fileSet");
		String pattern = fileSet.elementTextTrim("pattern");
		filePattern = Pattern.compile(pattern);
		File[] allFilesInDirectory = inputDirectory.listFiles();
		System.out.println("Number of Files to potentially match: " + allFilesInDirectory.length);
		for (File f : allFilesInDirectory ) {
			Matcher m = filePattern.matcher(f.getName());
			if ( m.matches() ) {
				filesToChange.add(f);
				System.out.println("pattern matched file: " + f.toString());	
			}
			
		}
		
		if ( filesToChange.size() == 0) {
			System.out.println("No files matched pattern: " + filePattern.toString());
		}
		
	}
	
	private void setInputDirectory() {
		String directory = filesElement.elementTextTrim("input_directory");
		if ( directory == null || directory.equalsIgnoreCase("this") || directory.equalsIgnoreCase(".")
				|| directory.length() == 0) {
			System.out.println("Input Directory: " + inputDirectory.toString()); //use current directory, user.dir
		}
		else {
			inputDirectory = new File(directory);
			if ( !inputDirectory.isDirectory() ) {
				System.out.println("Input Directory: " + directory + " does not exist or not a directory");
				System.exit(-1);
			}
		}
		System.out.println("Input Directory: " + inputDirectory.toString());	
	}
	
	public Document openFileToChange(File fileToChange) {
		Document fileDoc = null;
		try {
			SAXReader xmlReader = new SAXReader();
			fileDoc = xmlReader.read(fileToChange);

		} 
		catch (DocumentException de) {
			System.out.println("Error attempting to parse: " + fileToChange);
			de.printStackTrace(System.out);
			System.exit(-1);
		}
		
		return fileDoc;

	}
	
	public void createOutputDirectory() {
		String directory = filesElement.elementTextTrim("output_directory");
		if ( directory == null || directory.equalsIgnoreCase("this") || directory.equalsIgnoreCase(".")
				|| directory.length() == 0) {
			System.out.println("Output Directory: " + directoryToWriteFiles.toString());
			return; //use current directory, user.dir
		}
		directoryToWriteFiles = new File(directory);
		if ( !directoryToWriteFiles.exists() ) {
			directoryToWriteFiles.mkdir();
		}
		
		System.out.println("Output Directory: " + directoryToWriteFiles.toString());
	}
	
	public void writeModifiedFile(Document doc, String outputFileName) {
		File outputFile = new File(directoryToWriteFiles, outputFileName);
		writeModifiedFile(doc, outputFile);
	}
	
	public void writeModifiedFile(Document doc, File outputFile) {
		try {
			FileWriter fw = new FileWriter(outputFile);
			XMLWriter xmlWriter = new XMLWriter(fw);
			xmlWriter.write(doc);
			xmlWriter.close();
		} catch (IOException ioe) {
			System.out.println("Error attempting to open: " + outputFile.toString());
			ioe.printStackTrace(System.out);
			System.exit(-1);
		}
	}
}
